"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

"""
This is a port of the HelloWorld.ino from the tinkerkit LCD library.
It is implementing i2c.
"""

import sys
import time

from telemetrix import telemetrix

# TKLCD_Local lcd = TKLCD_Local(); // when programming a TKLCD module itself
# LiquidCrystal::begin(DEFAULT_SIZE_COL, DEFAULT_SIZE_ROWS)




# -*- coding: utf-8 -*-
"""
Created on Wed Jan 24 17:13:12 2024

@author: user01
"""

# -*- coding: utf-8 -*-
""" file DFRobot_RGBLCD1602.cpp
  # DFRobot_RGBLCD1602 class infrastructure, the implementation of basic methods
  @copyright	Copyright (c) 2010 DFRobot Co.Ltd (http://www.dfrobot.com)
  @licence     The MIT License (MIT)
  @maintainer [yangfeng](feng.yang@dfrobot.com)
  @version  V1.0
  @date  2021-09-24
  @url https://github.com/DFRobot/DFRobot_RGBLCD1602
"""
import time
import sys

# import wiringpi

LCD_ADDRESS = (0x7c >> 1)  # 0x3E or 62

# color define
WHITE = 0
RED = 1
GREEN = 2
BLUE = 3
REG_MODE1 = 0x00
REG_MODE2 = 0x01
REG_OUTPUT = 0x08
LCD_CLEARDISPLAY = 0x01
LCD_RETURNHOME = 0x02
LCD_ENTRYMODESET = 0x04
LCD_DISPLAYCONTROL = 0x08
LCD_CURSORSHIFT = 0x10
LCD_FUNCTIONSET = 0x20
LCD_SETCGRAMADDR = 0x40
LCD_SETDDRAMADDR = 0x80

# flags for display entry mode
LCD_ENTRYRIGHT = 0x00
LCD_ENTRYLEFT = 0x02
LCD_ENTRYSHIFTINCREMENT = 0x01
LCD_ENTRYSHIFTDECREMENT = 0x00

# flags for display on/off control
LCD_DISPLAYON = 0x04
LCD_DISPLAYOFF = 0x00
LCD_CURSORON = 0x02
LCD_CURSOROFF = 0x00
LCD_BLINKON = 0x01
LCD_BLINKOFF = 0x00

# flags for display/cursor shift
LCD_DISPLAYMOVE = 0x08
LCD_CURSORMOVE = 0x00
LCD_MOVERIGHT = 0x04
LCD_MOVELEFT = 0x00

# flags for function set
LCD_8BITMODE = 0x10
LCD_4BITMODE = 0x00
LCD_2LINE = 0x08
LCD_1LINE = 0x00
LCD_5x10DOTS = 0x04
LCD_5x8DOTS = 0x00

from telemetrix import telemetrix

"""
This file contains a library to control an i2c LCD. 
See i2c_ldc_pcf8574_hello_world.py for an example of its use.

Rewriting the scipt to capture DFRobot RGBLCD1602 v2 features - from the header file DFRobot_RGBLCD1602.h


/*
Change the RGBaddr value based on the hardware version
-----------------------------------------
       Moudule        | Version| RGBAddr|
-----------------------------------------
  LCD1602 Module      |  V1.0  | 0x60   |
-----------------------------------------
  LCD1602 Module      |  V1.1  | 0x6B   |
-----------------------------------------
  LCD1602 RGB Module  |  V1.0  | 0x60   |
-----------------------------------------
  LCD1602 RGB Module  |  V2.0  | 0x2D   |
-----------------------------------------

Scanning...
I2C device found at address 0x2D !
I2C device found at address 0x3E !
done


*/
"""


# self.board.i2c_write(self._address, [output])


class DFRobot_RGBLCD1602:
    def __init__(self, board, address=LCD_ADDRESS, columns=16, rows=2, address_rgb=0x2D):
        # def __init__(self, board, rgb_addr,col, row):
        # self.i2c=i2c
        self._row = rows
        self._col = columns
        self._address = address
        self.RGB_ADDRESS = address_rgb
        print("LCD _row=%d _col=%d" % (self._row, self._col))
        # self.LCD = board.(LCD_ADDRESS)
        self.board = board

        self.board.set_pin_mode_i2c()
        self.board.i2c_write(self._address, [0])

        if self.RGB_ADDRESS == 0x60:
            self.REG_RED = 0x04
            self.REG_GREEN = 0x03
            self.REG_BLUE = 0x02
            self.REG_ONLY = 0x02
        elif self.RGB_ADDRESS == (0x60 >> 1):
            self.REG_RED = 0x06
            self.REG_GREEN = 0x07
            self.REG_BLUE = 0x08
            self.REG_ONLY = 0x08
        elif self.RGB_ADDRESS == 0x6B:
            self.REG_RED = 0x06
            self.REG_GREEN = 0x05
            self.REG_BLUE = 0x04
            self.REG_ONLY = 0x04
        elif self.RGB_ADDRESS == 0x2D:
            self.REG_RED = 0x01
            self.REG_GREEN = 0x02
            self.REG_BLUE = 0x03
            self.REG_ONLY = 0x01
        self._show_function = LCD_4BITMODE | LCD_1LINE | LCD_5x8DOTS
        self._begin(self._row, self._col)

    def write(self, data):
        """
          @brief write character
          @param data the written data
        """
        b = bytearray(2)
        b[0] = 0x40
        b[1] = data
        self.board.i2c_write(self._address, [0x40, data])

    def set_RGB(self, r, g, b):
        """
          @brief set RGB
          @param r  red   range(0-255)
          @param g  green range(0-255)
          @param b  blue  range(0-255)
        """
        if self.RGB_ADDRESS == (0x60 >> 1):
            r = int(r * 192 / 255)
            g = int(g * 192 / 255)
            b = int(b * 192 / 255)

        self.board.i2c_write(self.RGB_ADDRESS, [self.REG_RED, r])
        self.board.i2c_write(self.RGB_ADDRESS, [self.REG_GREEN, g])
        self.board.i2c_write(self.RGB_ADDRESS, [self.REG_BLUE, b])
        if self.RGB_ADDRESS == 0x6b:
            self._set_reg(0x07, 0xFF)

    def set_cursor(self, col, row):
        """
          @brief set cursor position
          @param col columns optional range 0-15
          @param row rows optional range 0-1，0 is the first row, 1 is the second row
        """
        if row == 0:
            col |= 0x80
        else:
            col |= 0xc0
        self._command(col)

    def clear(self):
        """
          @brief clear the display and return the cursor to the initial position (position 0)
        """
        self._command(LCD_CLEARDISPLAY)
        time.sleep(0.002)

    def scroll_display_left(self):
        """
          @brief scroll left to display
        """
        self._command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT)

    def scroll_display_right(self):
        """
          @brief scroll right to display
        """
        self._command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT)

    def print_out(self, arg):
        """
          @brief output data to LCD to display
          @param arg output data
        """
        if isinstance(arg, int):
            arg = str(arg)

        for x in bytearray(arg, 'utf-8'):
            self.write(x)

    def home(self):
        """
          @brief return the cursor to the initial position（0,0）
        """
        self._command(LCD_RETURNHOME)  # set cursor position to zero
        time.sleep(1)  # this _command takes a long time!

    def no_display(self):
        """
          @brief Turn off the display
        """
        self._show_control &= ~LCD_DISPLAYON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def display(self):
        """
          @brief Turn on the display
        """
        self._show_control |= LCD_DISPLAYON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def stop_blink(self):
        """
          @brief Turn  off the blinking showCursor
        """
        self._show_control &= ~LCD_BLINKON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def blink(self):
        """
          @brief Turn on  the blinking showCursor
        """
        self._show_control |= LCD_BLINKON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def no_cursor(self):
        """
          @brief Turn off the underline showCursor 
        """
        self._show_control &= ~LCD_CURSORON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def cursor(self):
        """
          @brief Turn on the underline showCursor 
        """
        self._show_control |= LCD_CURSORON
        self._command(LCD_DISPLAYCONTROL | self._show_control)

    def left_to_right(self):
        """
          @brief This is for text that flows Left to Right
        """
        self._show_mode |= LCD_ENTRYLEFT
        self._command(LCD_ENTRYMODESET | self._show_mode)

    def right_to_left(self):
        """
          @brief This is for text that flows Right to Left
        """
        self._show_mode &= ~LCD_ENTRYLEFT
        self._command(LCD_ENTRYMODESET | self._show_mode)

    def no_autoscroll(self):
        """
          @brief This will 'left justify' text from the showCursor
        """
        self._show_mode &= ~LCD_ENTRYSHIFTINCREMENT
        self._command(LCD_ENTRYMODESET | self._show_mode)

    def autoscroll(self):
        """
          @brief This will 'right justify' text from the showCursor
        """
        self._show_mode |= LCD_ENTRYSHIFTINCREMENT
        self._command(LCD_ENTRYMODESET | self._show_mode)

    def custom_symbol(self, location, charmap):
        """
          @brief Allows us to fill the first 8 C
          @param location substitute character range（0-7）
          @param charmap  character listing the size is 8 bytes
        """
        location &= 0x7  # we only have 8 locations 0-7
        self._command(LCD_SETCGRAMADDR | (location << 3))
        for i in range(0, 8):
            # wiringpi.wiringPiI2CWriteReg8(self.LCD,0x40,charmap[i])
            self.board.i2c_write(self._address, 0x40, charmap[i])

    def set_backlight(self, mode):
        """
          @brief set the backlight
          @param mode  true indicates the backlight is turned on and set to white, false indicates the backlight is turned off
        """
        if mode:
            self.set_color_white()  # turn backlight on
        else:
            self.close_backlight()  # turn backlight off

    def printstr(self, c):
        """
          @brief output string
        """
        # /< This function is not identical to the function used for "real" I2C displays
        # /< it's here so the user sketch doesn't have to be changed 
        self.print_out(c)

    def set_pwm(self, color, pwm):
        """
           @brief set backlight PWM output
           @param color  backlight color  Preferences：REG_RED\REG_GREEN\REG_BLUE
           @param pwm  color intensity   range(0-255)
        """
        self._set_reg(color, pwm)
        if self.RGB_ADDRESS == 0x6b:
            self._set_reg(0x07, 0xFF)

    def set_color_white(self):
        """
           @brief set the backlight to white
        """
        self.set_RGB(255, 255, 255)

    def close_backlight(self):
        """
          @brief close the backlight
        """
        self.set_RGB(0, 0, 0)

    def _begin(self, cols, lines, dotsize=LCD_5x8DOTS):
        """
          @brief the initialization function
          @param cols columns optional range 0-15
          @param lines rows optional range 0-1，0 is the first row, 1 is the second row
          @param dotsize  character size LCD_5x8DOTS\LCD_5x10DOTS
        """
        if lines > 1:
            self._show_function |= LCD_2LINE

        self._num_lines = lines
        self._curr_line = 0

        # for some 1 line displays you can select a 10 pixel high font
        if (dotsize != 0) and (lines == 1):
            self._show_function |= LCD_5x10DOTS

            # SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!
        # according to datasheet, we need at least 40ms after power rises above 2.7V
        # before sending _commands. Arduino can turn on way before 4.5V, so we'll wait 50
        # delayMicroseconds(50000);
        time.sleep(0.05)

        # this is according to the hitachi HD44780 datasheet
        # page 45 figure 23

        # Send function set _command sequence
        self._command(LCD_FUNCTIONSET | self._show_function)
        # delayMicroseconds(4500);  # wait more than 4.1ms
        time.sleep(0.005)
        # second try
        self._command(LCD_FUNCTIONSET | self._show_function)
        # delayMicroseconds(150);
        time.sleep(0.005)
        # third go
        self._command(LCD_FUNCTIONSET | self._show_function)
        # finally, set # lines, font size, etc.
        self._command(LCD_FUNCTIONSET | self._show_function)
        # turn the display on with no cursor or blinking default
        self._show_control = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF
        self.display()
        # clear it off
        self.clear()
        # Initialize to default text direction (for romance languages)
        self._show_mode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT
        # set the entry mode
        self._command(LCD_ENTRYMODESET | self._show_mode)
        if self.RGB_ADDRESS == 0x60:
            # backlight init
            self._set_reg(REG_MODE1, 0)
            # set LEDs controllable by both PWM and GRPPWM registers
            self._set_reg(REG_OUTPUT, 0xFF)
            # set MODE2 values
            # 0010 0000 -> 0x20  (DMBLNK to 1, ie blinky mode)
            self._set_reg(REG_MODE2, 0x20)
        elif self.RGB_ADDRESS == (0x60 >> 1):
            self._set_reg(0x04, 0x15)
        elif self.RGB_ADDRESS == 0x6B:
            self._set_reg(0x2F, 0x00)
            self._set_reg(0x00, 0x20)
            self._set_reg(0x01, 0x00)
            self._set_reg(0x02, 0x01)
            self._set_reg(0x03, 0x04)
        self.set_color_white()

    def _command(self, cmd):
        """
          @brief send command
          @param data the sent data
        """
        b = bytearray(2)
        b[0] = 0x80
        b[1] = cmd
        # wiringpi.wiringPiI2CWriteReg8(self.LCD,0x80,cmd)
        self.board.i2c_write(self._address, [0x80, cmd])

    def _set_reg(self, reg, data):
        """
          @brief set the register
          @param addr register address
          @param data data
        """
        b = bytearray(1)
        b[0] = data
        # wiringpi.wiringPiI2CWriteReg8(self.RGB,reg,data)
        self.board.i2c_write(self._address, [reg, data])



def write_16_bit_i2c(register, value):
    """
    Write a 16 value to an i2c register
    :param register: i2c register
    :param value: 16 bit value
    :return: None
    """
    msb = value >> 8
    lsb = value & 0xff
    my_board.i2c_write(register, [msb, lsb])



import sys
import time
from telemetrix import telemetrix


class Cpu1:
    def __init__(self):
        # instantiate telemetrix
        self.board = telemetrix.Telemetrix()

        # set up pin 6 a digital output pin
        self.board.set_pin_mode_digital_output(6)

        # set up pin 9 for PWM output
        self.board.set_pin_mode_analog_output(9)

        # set up pin A2 for input
        self.board.set_pin_mode_analog_input(2, callback=self.the_callback)

        while True:
            # blink led on pin 6
            # wait for analog input
            try:
                self.board.digital_write(6, 1)
                time.sleep(.5)
                self.board.digital_write(6, 0)
                time.sleep(.5)
            except KeyboardInterrupt:
                self.board.shutdown()
                sys.exit(0)

    def the_callback(self, data):
        # data[2] contains the analog value
        # calculate a value within the range of 0 - 255
        # set pin 9 to this value
        self.board.analog_write(9, data[2] // 4)


Cpu1()



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import sys
import threading
import time
from collections import deque

import serial
# noinspection PyPackageRequirementscd
from serial.serialutil import SerialException
# noinspection PyPackageRequirements
from serial.tools import list_ports

from telemetrix.private_constants import PrivateConstants


# noinspection PyPep8,PyMethodMayBeStatic
class Telemetrix(threading.Thread):
    """
    This class exposes and implements the telemetrix API.
    It uses threading to accommodate concurrency.
    It includes the public API methods as well as
    a set of private methods.

    """

    # noinspection PyPep8,PyPep8,PyPep8
    def __init__(self, com_port=None, arduino_instance_id=1,
                 arduino_wait=4, sleep_tune=0.000001,
                 shutdown_on_exception=True):

        """

        :param com_port: e.g. COM3 or /dev/ttyACM0.
                         Only use if you wish to bypass auto com port
                         detection.

        :param arduino_instance_id: Match with the value installed on the
                                    arduino-telemetrix sketch.

        :param arduino_wait: Amount of time to wait for an Arduino to
                             fully reset itself.

        :param sleep_tune: A tuning parameter (typically not changed by user)

        :param shutdown_on_exception: call shutdown before raising
                                      a RunTimeError exception, or
                                      receiving a KeyboardInterrupt exception
        """

        # initialize threading parent
        threading.Thread.__init__(self)

        # create the threads and set them as daemons so
        # that they stop when the program is closed

        # create a thread to interpret received serial data
        self.the_reporter_thread = threading.Thread(target=self._reporter)
        self.the_reporter_thread.daemon = True

        self.the_data_receive_thread = threading.Thread(target=self._serial_receiver)

        self.the_data_receive_thread.daemon = True

        # flag to allow the reporter and receive threads to run.
        self.run_event = threading.Event()

        # check to make sure that Python interpreter is version 3.7 or greater
        python_version = sys.version_info
        if python_version[0] >= 3:
            if python_version[1] >= 7:
                pass
            else:
                raise RuntimeError("ERROR: Python 3.7 or greater is "
                                   "required for use of this program.")

        # save input parameters as instance variables
        self.com_port = com_port
        self.arduino_instance_id = arduino_instance_id
        self.arduino_wait = arduino_wait
        self.sleep_tune = sleep_tune
        self.shutdown_on_exception = shutdown_on_exception

        # create a deque to receive and process data from the arduino
        self.the_deque = deque()

        # The report_dispatch dictionary is used to process
        # incoming report messages by looking up the report message
        # and executing its associated processing method.
        # The value following the method is the number of bytes to
        # retrieve from the deque to process the report (not including report id).
        self.report_dispatch = {}

        # To add a command to the command dispatch table, append here.
        self.report_dispatch.update({PrivateConstants.LOOP_COMMAND: [self._report_loop_data, 1]})
        self.report_dispatch.update({PrivateConstants.DEBUG_PRINT: [self._report_debug_data, 3]})
        self.report_dispatch.update({PrivateConstants.DIGITAL_REPORT: [self._digital_message, 2]})
        self.report_dispatch.update({PrivateConstants.ANALOG_REPORT: [self._analog_message, 3]})
        self.report_dispatch.update({PrivateConstants.FIRMWARE_REPORT: [self._firmware_message, 2]})
        self.report_dispatch.update({PrivateConstants.I_AM_HERE_REPORT: [self._i_am_here, 1]})
        self.report_dispatch.update({PrivateConstants.SERVO_UNAVAILABLE: [self._servo_unavailable, 1]})

        # dictionaries to store the callbacks for each pin
        self.analog_callbacks = {}

        self.digital_callbacks = {}

        # serial port in use
        self.serial_port = None

        # flag to indicate we are in shutdown mode
        self.shutdown_flag = False

        # debug loopback callback method
        self.loop_back_callback = None

        # flag to indicate the start of a new report
        self.new_report_start = True

        # firmware version to be stored here
        self.firmware_version = []

        # reported arduino instance id
        self.reported_arduino_id = []

        self.the_reporter_thread.start()
        self.the_data_receive_thread.start()

        print(f"Telemetrix:  Version {PrivateConstants.TELEMETRIX_VERSION}\n\n"
              f"Copyright (c) 2020 Alan Yorinks All Rights Reserved.\n")

        if not self.com_port:
            # user did not specify a com_port
            try:
                self._find_arduino()
            except KeyboardInterrupt:
                if self.shutdown_on_exception:
                    self.shutdown()
        else:
            # com_port specified - set com_port and baud rate
            try:
                self._manual_open()
            except KeyboardInterrupt:
                if self.shutdown_on_exception:
                    self.shutdown()

        if self.serial_port:
            print(f"Arduino compatible device found and connected to {self.serial_port.port}")

            self.serial_port.reset_input_buffer()
            self.serial_port.reset_output_buffer()

        # no com_port found - raise a runtime exception
        else:
            if self.shutdown_on_exception:
                self.shutdown()
            raise RuntimeError('No Arduino Found or User Aborted Program')

        # allow the threads to run
        self._run_threads()
        print(f'Waiting for Arduino to reset')
        print(f'Reset Complete')

        print('Retrieving Arduino ID...')
        self._get_arduino_id()

        if self.reported_arduino_id != self.arduino_instance_id:
            raise RuntimeError(f'Incorrect Arduino ID: {self.reported_arduino_id}')
        print('Valid Arduino ID Found.')
        # get arduino firmware version and print it
        print('\nRetrieving arduino-telemetrix firmware ID...')
        self._get_firmware_version()

        if not self.firmware_version:
            if self.shutdown_on_exception:
                self.shutdown()
            raise RuntimeError(f'Firmata Sketch Firmware Version Not Found')

        else:
            print(f'arduino-telemetrix firmware version: {self.firmware_version[0]}.'
                  f'{self.firmware_version[1]}')

    def _find_arduino(self):
        """
        This method will search all potential serial ports for an Arduino
        containing a sketch that has a matching arduino_instance_id as
        specified in the input parameters of this class.

        This is used explicitly with the FirmataExpress sketch.
        """

        # a list of serial ports to be checked
        serial_ports = []

        print('Opening all potential serial ports...')
        the_ports_list = list_ports.comports()
        for port in the_ports_list:
            if port.pid is None:
                continue
            try:
                self.serial_port = serial.Serial(port.device, 115200,
                                                 timeout=1, writeTimeout=0)
            except SerialException:
                continue
            # create a list of serial ports that we opened
            serial_ports.append(self.serial_port)

            # display to the user
            print('\t' + port.device)

            # clear out any possible data in the input buffer
        # wait for arduino to reset
        print(f'\nWaiting {self.arduino_wait} seconds(arduino_wait) for Arduino devices to '
              'reset...')
        # temporary for testing
        time.sleep(self.arduino_wait)

        # check for correct arduino device
        self.serial_port.reset_input_buffer()
        self.serial_port.reset_output_buffer()

    def _manual_open(self):
        """
        Com port was specified by the user - try to open up that port

        """
        # if port is not found, a serial exception will be thrown
        try:
            print(f'Opening {self.com_port}...')
            self.serial_port = serial.Serial(self.com_port, 115200,
                                             timeout=1, writeTimeout=0)

            print(f'\nWaiting {self.arduino_wait} seconds(arduino_wait) for Arduino devices to '
                  'reset...')
            time.sleep(self.arduino_wait)

            self._get_arduino_id()

            if self.reported_arduino_id != self.arduino_instance_id:
                raise RuntimeError(f'Incorrect Arduino ID: {self.reported_arduino_id}')
            print('Valid Arduino ID Found.')
            # get arduino firmware version and print it
            print('\nRetrieving arduino-telemetrix firmware ID...')
            self._get_firmware_version()

            if not self.firmware_version:
                if self.shutdown_on_exception:
                    self.shutdown()
                raise RuntimeError(f'Firmata Sketch Firmware Version Not Found')

            else:
                print(f'arduino-telemetrix firmware version: {self.firmware_version[0]}.'
                      f'{self.firmware_version[1]}')
        except KeyboardInterrupt:
            raise RuntimeError('User Hit Control-C')

    def analog_read(self, pin):
        """
        Retrieve the last data update for the specified analog pin.

        :param pin: Analog pin number (ex. A2 is specified as 2)

        :returns: A list = [last value change,  time_stamp]
        """
        raise NotImplementedError

    def analog_write(self, pin, value):
        """
        Set the specified pin to the specified value.

        :param pin: arduino pin number

        :param value: pin value (1 or 0)

        """
        command = (PrivateConstants.ANALOG_WRITE, pin, value)
        self._send_command(command)

    def digital_read(self, pin):
        """
        Retrieve the last data update for the specified digital pin.

        :param pin: Digital pin number

        :returns: A list = [last value change,  time_stamp]

        """
        raise NotImplementedError

    def digital_pin_write(self, pin, value):
        """
        Set the specified pin to the specified value directly without port manipulation.

        :param pin: arduino pin number

        :param value: pin value

        """

        command = (PrivateConstants.DIGITAL_WRITE, pin, value)
        self._send_command(command)

    def digital_write(self, pin, value):
        """
        Set the specified pin to the specified value.

        :param pin: arduino pin number

        :param value: pin value (1 or 0)

        """

        command = (PrivateConstants.DIGITAL_WRITE, pin, value)
        self._send_command(command)

    def disable_all_reporting(self):
        """
        Disable reporting for all digital and analog input pins
        """
        command = (PrivateConstants.MODIFY_REPORTING, PrivateConstants.REPORTING_DISABLE_ALL, 0)
        self._send_command(command)

    def disable_analog_reporting(self, pin):
        """
        Disables analog reporting for a single analog pin.

        :param pin: Analog pin number. For example for A0, the number is 0.

        """
        command = (PrivateConstants.MODIFY_REPORTING, PrivateConstants.REPORTING_ANALOG_DISABLE, pin)
        self._send_command(command)

    def disable_digital_reporting(self, pin):
        """
        Disables digital reporting. By turning reporting off for this pin,
        Reporting is disabled for all 8 bits in the "port"

        :param pin: Pin and all pins for this port

        """
        command = (PrivateConstants.MODIFY_REPORTING, PrivateConstants.REPORTING_DIGITAL_DISABLE, pin)
        self._send_command(command)

    def enable_analog_reporting(self, pin):
        """
        Enables analog reporting. This is an alias for set_pin_mode_analog_input.
        Disabling analog reporting sets the pin to a digital input pin,
        so we need to provide the callback and differential if we wish
        to specify it.

        :param pin: Analog pin number. For example for A0, the number is 0.


        """
        command = [PrivateConstants.MODIFY_REPORTING, PrivateConstants.REPORTING_ANALOG_ENABLE, pin]
        self._send_command(command)

    def enable_digital_reporting(self, pin):
        """
        Enables digital reporting. By turning reporting on for all 8 bits
        in the "port" - this is part of Firmata's protocol specification.

        :param pin: Pin and all pins for this port
        """

        command = [PrivateConstants.MODIFY_REPORTING, PrivateConstants.REPORTING_DIGITAL_ENABLE, pin]
        self._send_command(command)

    def _get_arduino_id(self):
        """
        Retrieve arduino-telemetrix arduino id
        :return:
        """
        command = [PrivateConstants.ARE_U_THERE]
        self._send_command(command)
        # provide time for the reply
        time.sleep(.1)

    def _get_firmware_version(self):
        """
        This method retrieves the
        arduino-telemetrix firmware version

        :returns: Firmata firmware version
        """
        command = [PrivateConstants.GET_FIRMWARE_VERSION]
        self._send_command(command)
        # provide time for the reply
        time.sleep(.1)

    def loop_back(self, start_character, callback=None):
        """
        This is a debugging method to send a character to the
        Arduino device, and have the device loop it back.
        :param start_character: The character to loop back. It should be
                                an integer.
        :param callback: Looped back character will appear in the callback method
        """
        command = [PrivateConstants.LOOP_COMMAND, ord(start_character)]
        self.loop_back_callback = callback
        self._send_command(command)

    def set_pin_mode_analog_output(self, pin_number):
        """
        Set a pin as a pwm (analog output) pin.

        :param pin_number:arduino pin number

        """
        self._set_pin_mode(pin_number, PrivateConstants.AT_OUTPUT)

    def set_pin_mode_analog_input(self, pin_number, callback=None):
        """
        Set a pin as an analog input.

        :param pin_number: arduino pin number

        :param callback: callback function


        callback returns a data list:

        [pin_type, pin_number, pin_value, raw_time_stamp]

        The pin_type for analog input pins = 2

        """
        self._set_pin_mode(pin_number, PrivateConstants.AT_ANALOG,
                           callback=callback)

    def set_pin_mode_digital_input(self, pin_number, callback=None):
        """
        Set a pin as a digital input.

        :param pin_number: arduino pin number

        :param callback: callback function


        callback returns a data list:

        [pin_type, pin_number, pin_value, raw_time_stamp]

        The pin_type for digital input pins = 0

        """
        self._set_pin_mode(pin_number, PrivateConstants.AT_INPUT, callback)

    def set_pin_mode_digital_input_pullup(self, pin_number, callback=None):
        """
        Set a pin as a digital input with pullup enabled.

        :param pin_number: arduino pin number

        :param callback: callback function


        callback returns a data list:

        [pin_type, pin_number, pin_value, raw_time_stamp]

        The pin_type for digital input pins with pullups enabled = 11

        """
        self._set_pin_mode(pin_number, PrivateConstants.AT_INPUT_PULLUP, callback)

    def set_pin_mode_digital_output(self, pin_number):
        """
        Set a pin as a digital output pin.

        :param pin_number: arduino pin number
        """

        self._set_pin_mode(pin_number, PrivateConstants.AT_OUTPUT)

    def set_pin_mode_servo(self, pin_number, min_pulse=544, max_pulse=2400):
        """
        Attach a pin to a servo motor
        :param pin_number: pin
        :param min_pulse: minimum pulse width
        :param max_pulse: maximum pulse width
        """
        minv = (min_pulse).to_bytes(2, byteorder="big")
        maxv = (max_pulse).to_bytes(2, byteorder="big")

        command = [PrivateConstants.SERVO_ATTACH, pin_number,
                   minv[0], minv[1], maxv[0], maxv[1]]
        self._send_command(command)

    def servo_write(self, pin_number, angle):
        """
        Set a servo attached to a pin to a given angle
        :param pin_number: pin
        :param angle: angle (0-180)
        """
        command = [PrivateConstants.SERVO_WRITE, pin_number, angle]
        self._send_command(command)

    def servo_detach(self, pin_number):
        command = [PrivateConstants.SERVO_DETACH, pin_number]
        self._send_command(command)

    def _set_pin_mode(self, pin_number, pin_state, callback=None):

        """
        A private method to set the various pin modes.

        :param pin_number: arduino pin number

        :param pin_state: INPUT/OUTPUT/ANALOG/PWM/PULLUP
                         For SERVO use: set_pin_mode_servo
                         For DHT   use: set_pin_mode_dht

        :param callback: A reference to a call back function to be
                         called when pin data value changes

        """
        # command = []
        if callback:
            if pin_state == PrivateConstants.AT_INPUT:
                self.digital_callbacks[pin_number] = callback
            elif pin_state == PrivateConstants.AT_INPUT_PULLUP:
                self.digital_callbacks[pin_number] = callback
            elif pin_state == PrivateConstants.AT_ANALOG:
                self.analog_callbacks[pin_number] = callback
            else:
                print('{} {}'.format('set_pin_mode: callback ignored for '
                                     'pin state:', pin_state))

        if pin_state == PrivateConstants.AT_INPUT:
            command = [PrivateConstants.SET_PIN_MODE, pin_number, PrivateConstants.AT_INPUT, 1]

        elif pin_state == PrivateConstants.AT_INPUT_PULLUP:
            command = [PrivateConstants.SET_PIN_MODE, pin_number, PrivateConstants.AT_INPUT_PULLUP, 1]

        elif pin_state == PrivateConstants.AT_OUTPUT:
            command = [PrivateConstants.SET_PIN_MODE, pin_number, PrivateConstants.AT_OUTPUT, 0, 0]

        elif pin_state == PrivateConstants.AT_ANALOG:
            command = [PrivateConstants.SET_PIN_MODE, pin_number, PrivateConstants.AT_ANALOG, 1]
        else:
            raise RuntimeError('Unknown pin state')

        if command:
            self._send_command(command)

    def shutdown(self):
        """
        This method attempts an orderly shutdown
        If any exceptions are thrown, they are ignored.
        """
        self.shutdown_flag = True

        self._stop_threads()

        try:
            self.disable_all_reporting()

            self.serial_port.reset_input_buffer()
            self.serial_port.close()

        except (RuntimeError, SerialException, OSError):
            # ignore error on shutdown
            pass

    '''
    report message handlers
    '''

    def _analog_message(self, data):
        """
        This is a private message handler method.
        It is a message handler for analog messages.

        :param data: message data

        """
        pin = data[0]
        value = (data[1] << 8) + data[2]
        # set the current value in the pin structure
        time_stamp = time.time()
        # self.digital_pins[pin].event_time = time_stamp
        if self.analog_callbacks[pin]:
            message = [PrivateConstants.ANALOG_REPORT, pin, value, time_stamp]
            self.analog_callbacks[pin](message)

    def _digital_message(self, data):
        """
        This is a private message handler method.
        It is a message handler for Digital Messages.

        :param data: digital message

        """
        pin = data[0]
        value = data[1]

        time_stamp = time.time()
        if self.digital_callbacks[pin]:
            message = [PrivateConstants.DIGITAL_REPORT, pin, value, time_stamp]
            self.digital_callbacks[pin](message)

    def _firmware_message(self, data):
        """
        Telemetrix4Arduino firmware version message
        :param data: data[0] = major number, data[1] = minor number
        """

        self.firmware_version = [data[0], data[1]]

    def _i_am_here(self, data):
        """
        Reply to are_u_there message
        :param data: arduino id
        """
        self.reported_arduino_id = data[0]

    def _report_debug_data(self, data):
        """
        Print debug data sent from Arduino
        :param data: data[0] is a byte followed by 2
                     bytes that comprise an integer
        :return:
        """
        value = (data[1] << 8) + data[2]
        print(f'DEBUG ID: {data[0]} Value: {value}')

    def _report_loop_data(self, data):
        """
        Print data that was looped back
        :param data: byte of loop back data
        :return:
        """
        if self.loop_back_callback:
            self.loop_back_callback(data)

    def _send_command(self, command):
        """
        This is a private utility method.
        The method sends a non-sysex command to Firmata.

        :param command:  command data

        :returns: number of bytes sent
        """
        # send_message = ""
        # print('command: ', command)
        send_message = bytes(command)
        try:
            self.serial_port.write(send_message)
            # print(send_message)
        except SerialException:
            if self.shutdown_on_exception:
                self.shutdown()
            raise RuntimeError('write fail in _send_command')

    def _servo_unavailable(self, report):
        """
        Message if no servos are available for use.
        :param report: pin number
        """
        raise RuntimeError(f'Servo Attach For Pin {report[0]} Failed: No Available Servos')

    def _run_threads(self):
        self.run_event.set()

    def _is_running(self):
        return self.run_event.is_set()

    def _stop_threads(self):
        self.run_event.clear()

    def _reporter(self):
        """
        This is the reporter thread. It continuously pulls data from
        the deque. When a full message is detected, that message is
        processed.
        """
        self.run_event.wait()
        num_args = 0
        method = None

        while self._is_running() and not self.shutdown_flag:
            if len(self.the_deque):

                # get next byte from the deque and process it
                data = self.the_deque.popleft()
                # print('data ', data)

                # this list will be populated with the received data for the report
                response_data = []

                # if self.new_report_start:
                dispatch_entry = self.report_dispatch.get(data)

                # this calls the method retrieved from the dispatch table
                try:
                    method = dispatch_entry[0]
                except TypeError:
                    pass

                # get the number of parameters that this command provides
                try:
                    num_args = dispatch_entry[1]
                except TypeError:
                    pass

                # look at the number of args that the selected method requires
                # now get that number of bytes to pass to the called method

                for i in range(num_args):
                    while len(self.the_deque) == 0:
                        pass
                    data = self.the_deque.popleft()
                    response_data.append(data)
                    # go execute the command with the argument list
                method(response_data)
                continue
            else:
                time.sleep(self.sleep_tune)

    def _serial_receiver(self):
        """
        Thread to continuously check for incoming data.
        When a byte comes in, place it onto the deque.
        """
        self.run_event.wait()

        while self._is_running() and not self.shutdown_flag:
            # we can get an OSError: [Errno9] Bad file descriptor when shutting down
            # just ignore it
            try:
                if self.serial_port.inWaiting():
                    c = self.serial_port.read()
                    self.the_deque.append(ord(c))
                    # print(ord(c))
                else:
                    time.sleep(self.sleep_tune)
                    # continue
            except OSError:
                pass



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import sys
import time

from telemetrix import telemetrix

"""
Attach a pin to a servo and move it about.
"""

# some globals
SERVO_PIN = 4

# Create a Telemetrix instance.
board = telemetrix.Telemetrix(ip_address='192.168.2.168')
board.set_pin_mode_servo(SERVO_PIN)
time.sleep(.2)
board.servo_write(SERVO_PIN, 90)
time.sleep(1)
board.servo_write(SERVO_PIN, 0)
time.sleep(1)
board.servo_write(SERVO_PIN, 180)
time.sleep(1)
board.servo_write(SERVO_PIN, 90)
time.sleep(1)

board.servo_detach(SERVO_PIN)
time.sleep(.2)
board.shutdown()



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

"""
This file verifies the set and get methods for the stepper API.
"""
import time
from telemetrix import telemetrix


# Create a Telemetrix instance.
board = telemetrix.Telemetrix()


def current_position_callback(data):
    print(f'current_position_callback returns {data[2]}\n')


def target_position_callback(data):
    print(f'target_position_callback returns {data[2]}')


def distance_to_go_callback(data):
    print(f'distance_to_go_callback returns {data[2]}\n')


def is_running_callback(data):
    print(f'is_running_callback returns {data[1]}\n')


# create an accelstepper instance for a TB6600 motor driver
motor = board.set_pin_mode_stepper(interface=1, pin1=7, pin2=8)
print('Checking if board is running:')
board.stepper_is_running(motor, is_running_callback)
time.sleep(.2)


print('\nSetting current position to 216')
board.stepper_set_current_position(motor, 216)
board.stepper_get_current_position(motor, current_position_callback)

time.sleep(.2)

print('Setting move_to target to 300 (absolute motion.)')
board.stepper_move_to(motor, 300)
board.stepper_get_target_position(motor, target_position_callback)
time.sleep(.2)
board.stepper_get_distance_to_go(motor, distance_to_go_callback)
time.sleep(.2)

print('Setting move_to target to 300 (relative motion.)')
board.stepper_move(motor, 300)
board.stepper_get_target_position(motor, target_position_callback)
time.sleep(.2)
board.stepper_get_distance_to_go(motor, distance_to_go_callback)
time.sleep(.2)

print('Because we set all the parameters to move the motor, but have not started')
print('it yet, IS_RUNNING reports that the motor is in motion, even though it is not')
print('actually moving.')
board.stepper_is_running(motor, is_running_callback)
time.sleep(.2)



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import sys
import time

from telemetrix import telemetrix

"""
Run a motor to an absolute position. Server will send a callback notification 
when motion is complete.
"""

# global variables
current_position_complete = False
current_position = None

distance_to_go_complete = False
distance_to_go = None


def distance_to_go_callback(data):
    global distance_to_go_complete, distance_to_go
    distance_to_go = data[2]
    distance_to_go_complete = True


def completion_callback(data):
    print(f'complete: {data}')


def get_current_position(data):
    # print(f' info {data}')
    global current_position, current_position_complete
    current_position = data[2]
    current_position_complete = True


def setup(the_motor, the_board):
    global current_position, current_position_complete
    the_board.stepper_get_current_position(the_motor, get_current_position)
    while not current_position_complete:
        time.sleep(.0001)
    current_position_complete = False
    print(f'Setup: current position = {current_position}')

    the_board.stepper_set_max_speed(the_motor, 1000)
    the_board.stepper_set_acceleration(the_motor, 100)
    the_board.stepper_set_speed(the_motor, 200)
    the_board.stepper_move_to(the_motor, 1024)


def loop(the_motor, the_board):
    global current_position, current_position_complete
    global distance_to_go_complete, distance_to_go
    endpoint = 1024

    while True:
        the_board.stepper_get_distance_to_go(the_motor, distance_to_go_callback)
        while not distance_to_go_complete:
            time.sleep(.0000001)
        distance_to_go_complete = False

        if distance_to_go == 0:
            the_board.stepper_set_current_position(the_motor, 0)
            endpoint = -endpoint
            the_board.stepper_move_to(the_motor, endpoint)
        the_board.stepper_run(the_motor, completion_callback)


# Create a Telemetrix instance.
board = telemetrix.Telemetrix(arduino_wait=4)
motor = board.set_pin_mode_stepper(interface=8, pin1=8, pin2=9, pin3=10, pin4=11)
setup(motor, board)
loop(motor, board)



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import sys
import time

from telemetrix import telemetrix

"""
Run a motor to a relative position.
"""

# Create a Telemetrix instance.
board = telemetrix.Telemetrix()


def the_callback(data):
    date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[2]))
    print(f'Motor {data[1]} relative  motion completed at: {date}.')


# create an accelstepper instance for a TB6600 motor driver
# motor = board.set_pin_mode_stepper(interface=1, pin1=7, pin2=8)

# if you are using a 28BYJ-48 Stepper Motor with ULN2003
# comment out the line above and uncomment out the line below.
motor1 = board.set_pin_mode_stepper(interface=8, pin1=8, pin2=10, pin3=9, pin4=11)
motor2 = board.set_pin_mode_stepper(interface=8, pin1=2, pin2=3, pin3=4, pin4=5)


# set the max speed and acceleration
board.stepper_set_max_speed(motor1, 400)
board.stepper_set_max_speed(motor1, 400)

board.stepper_set_acceleration(motor1, 800)
board.stepper_set_acceleration(motor2, 800)


# set the relative position in steps
board.stepper_move(motor1, 2000)
board.stepper_move(motor2, 2000)


# run the motor
board.stepper_run(motor1, completion_callback=the_callback)
board.stepper_run(motor2, completion_callback=the_callback)


# keep application running
while True:
    try:
        time.sleep(.2)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)



"""
 Copyright (c) 2022 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import time
import sys
from telemetrix_rpi_pico_w import telemetrix_rpi_pico_w

"""
Configure a BME280 for SPI operation and return a raw temperature in degrees C.

This is a vary simple demo and does not take advantage of all of the BME380 features.
It does not pretend to be accurate, but only demonstrates using the SPI api calls.
"""

# Bme280RegisterAddressConfig = 0xF5
bme_temp_register = 0xfa
bme_reset_register = 0xe0

# data[0] = 13 - SPI report
# data[1] = SPI Port
# data[2] = BME register
# data[3 ...] = data returned


def device_callback(data):
    # temperature is returned in 3 bytes
    # we convert
    if data[2] == bme_temp_register:
        temp = bytes(data[3:6])
        int_val = int.from_bytes(temp, "big") / 10000
        print(f'Temp={int_val} C')
    else:
        print('Unknown register')


# instantiate an instance of TelemetrixRpiPicoW
board = telemetrix_rpi_pico_w.TelemetrixRpiPicoW(ip_address='192.168.2.102')

# set the pin mode for SPI0.
board.set_pin_mode_spi(spi_port=1)

# reset the bme280
# the data to write must be presented in the form of a list
board.spi_write_blocking([bme_reset_register], spi_port=1)

# after resetting, we must call set pin mode again to start a new SPI.begin()
board.set_pin_mode_spi(spi_port=1)

# get the raw temperature
board.spi_read_blocking(bme_temp_register, 3, spi_port=1, call_back=device_callback)

# provide time for the data to return
try:
    time.sleep(.5)
    board.shutdown()
except KeyboardInterrupt:
    board.shutdown()



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys
import time
from telemetrix import telemetrix

"""
This program monitors a DHT 22 sensor. 
"""

# indices into callback data for valid data
# REPORT_TYPE = 0
# ERROR_CODE = 1  # value of 0 is no errors
# PIN = 2
# DHT_TYP = 3
# HUMIDITY = 4
# TEMPERATURE = 5
# TIME = 6

# indices into callback data for error report
# REPORT_TYPE = 0
# ERROR_CODE = 1 (value of 0 is no errors)
# PIN = 2
# DHT_TYPE = 3
# TIME = 4


# A callback function to display the distance
# noinspection GrazieInspection
def the_callback(data):
    """
    The callback function to display the change in distance
    :param data: [report_type = PrivateConstants.DHT, pin number, humidity, temperature timestamp]
                 if this is an error report:
                 [report_type = PrivateConstants.DHT, pin number, error value timestamp]
    """
    if data[1]:
        # error message
        date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[4]))
        print(f'DHT Error Report:'
              f'Pin: {data[3]} Error: {data[2]}  Time: {date}')
    else:
        date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[6]))
        print(f'DHT Valid Data Report:'
              f'Pin: {data[3]} Humidity: {data[4]} Temperature: {data[5]} Time: {date}')


def dht(my_board):
    # noinspection GrazieInspection
    """
        Set the pin mode for a DHT 22 device. Results will appear via the
        callback.

        :param my_board: a telemetrix instance

        """

    # set the pin mode for the trigger and echo pins
    my_board.set_pin_mode_dht(5, callback=the_callback, dht_type=22)

    # wait forever
    while True:
        try:
            time.sleep(.01)
        except KeyboardInterrupt:
            my_board.shutdown()
            sys.exit(0)


board = telemetrix.Telemetrix(ip_address='192.168.2.112')
try:
    dht(board)
    board.shutdown()
except (KeyboardInterrupt, RuntimeError):
    board.shutdown()
    sys.exit(0)



class Play:
    def __init__(self):
        self.rs = 0
        self.rw = 0
        self.E = 0
        self.Led = 1
        self.data = 72

    def GetLowData(self, e):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= e << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0x0F) << 4

        print(buffer)

    def GetHighData(self, e):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= e << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0x0F)

        print(buffer)


z = Play()
z.GetHighData(1)
z.GetHighData(0)

z.GetLowData(1)
z.GetLowData(0)



"""
 Copyright (c) 2015-2019 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


class PrivateConstants:
    """
    This class contains a set of constants for telemetrix internal use .
    """

    # commands
    # send a loop back request - for debugging communications
    LOOP_COMMAND = 0
    SET_PIN_MODE = 1  # set a pin to INPUT/OUTPUT/PWM/etc
    DIGITAL_WRITE = 2  # set a single digital pin value instead of entire port
    ANALOG_WRITE = 3
    MODIFY_REPORTING = 4
    GET_FIRMWARE_VERSION = 5
    ARE_U_THERE = 6  # Arduino ID query for auto-detect of telemetrix connected boards
    SERVO_ATTACH = 7
    SERVO_WRITE = 8
    SERVO_DETACH = 9

    # reports
    # debug data from Arduino
    DIGITAL_REPORT = DIGITAL_WRITE
    ANALOG_REPORT = ANALOG_WRITE
    FIRMWARE_REPORT = GET_FIRMWARE_VERSION
    I_AM_HERE_REPORT = ARE_U_THERE
    SERVO_UNAVAILABLE = SERVO_ATTACH
    DEBUG_PRINT = 99

    TELEMETRIX_VERSION = "0.01"

    # reporting control
    REPORTING_DISABLE_ALL = 0
    REPORTING_ANALOG_ENABLE = 1
    REPORTING_DIGITAL_ENABLE = 2
    REPORTING_ANALOG_DISABLE = 3
    REPORTING_DIGITAL_DISABLE = 4

    # Pin mode definitions
    AT_INPUT = 0
    AT_OUTPUT = 1
    AT_INPUT_PULLUP = 2
    AT_ANALOG = 3
    AT_SERVO = 4
    AT_SONAR = 5
    AT_DHT = 6
    AT_MODE_NOT_SET = 255

    # maximum number of digital pins supported
    NUMBER_OF_DIGITAL_PINS = 100

    # maximum number of analog pins supported
    NUMBER_OF_ANALOG_PINS = 20



import sys
import time

from telemetrix import telemetrix


class ArmServo:
    def __init__ (self, pin, pos=90):
        board.set_pin_mode_servo(pin)
        board.servo_write(pin, pos)

        self.pin = pin
        self.pos = pos

    def read(self):
        return self.pos

    def write(self, new_pos, servo_delay=0.0025):
        print("writing")
        curr_pos = self.pos
        if new_pos == curr_pos:
            return
        elif new_pos < curr_pos:
            for pos in range(curr_pos, new_pos-1, -1):
                board.servo_write(self.pin, pos)
                sleep(servo_delay)
                self.pos = pos
        else:
            for pos in range(curr_pos, new_pos+1):
                board.servo_write(self.pin, pos)
                sleep(servo_delay)
                self.pos = pos

board = telemetrix.Telemetrix()

claw = ArmServo(2)           # red
wrist_rotate = ArmServo(3)   # orange
wrist_up_down = ArmServo(4)  # yellow
elbow = ArmServo(5)          # green
shoulder = ArmServo(6)       # blue
base = ArmServo(7)           # violet


"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import sys
import time

from telemetrix import telemetrix

"""
Setup a pin for digital output 
and toggle the pin 5 times.
"""

# some globals
DIGITAL_PIN = 6  # the board LED

# Create a Telemetrix instance.
board = telemetrix.Telemetrix()

# Set the DIGITAL_PIN as an output pin
board.set_pin_mode_digital_output(DIGITAL_PIN)

# Blink the LED and provide feedback as
# to the LED state on the console.
start_time = time.time()
for blink in range(1000):
    # When hitting control-c to end the program
    # in this loop, we are likely to get a KeyboardInterrupt
    # exception. Catch the exception and exit gracefully.
    try:
        # print('1')
        board.digital_write(DIGITAL_PIN, 1)
        time.sleep(.00001)
        #print('0')
        board.digital_write(DIGITAL_PIN, 0)
        time.sleep(.00001)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)


print(f'1000 iterations took {time.time() - start_time} seconds.')
board.shutdown()



import sys
import time
from telemetrix import telemetrix
from i2c_lcd_pcf8574 import LCD_I2C

"""
This example sets up and control an pcf8574 i2c LCD display.
"""


def pcf8574(board):
    lcd = LCD_I2C(board)
    lcd.begin()
    for i in range(3):
        lcd.backlight()
        time.sleep(1)
        lcd.noBacklight()
        time.sleep(1)


board = telemetrix.Telemetrix()
try:
    pcf8574(board)
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)
    


import sys
import time
from telemetrix import telemetrix

mcu = None
try:
    mcu = telemetrix.Telemetrix(sleep_tune=0.1)
    # mcu = telemetrix.Telemetrix(sleep_tune=0.000001)
    while True:
        try:
            time.sleep(.5)
        except KeyboardInterrupt:
            mcu.shutdown()
            sys.exit(0)
except KeyboardInterrupt:
    mcu.shutdown()
    sys.exit(0)



# -*- coding: utf-8 -*-
"""
Created on Wed Jan 24 17:14:06 2024

@author: user01
"""

# -*- coding: utf-8 -*-
'''file hello_world.py
 # @brief display "hello,world!"
 # @copyright	Copyright (c) 2010 DFRobot Co.Ltd (http://www.dfrobot.com)
 # @licence     The MIT License (MIT)
 # @maintainer [yangfeng](feng.yang@dfrobot.com)
 # @version  V1.0
 # @date  2021-09-26
 # @url https://github.com/DFRobot/DFRobot_RGBLCD1602
'''
import sys
sys.path.append('../')
from DFRobot_RGBLCD1602_telemetrix import *
import time
from telemetrix import telemetrix

colorR = 255
colorG = 0
colorB = 0
# Change the rgb_addr value based on the hardware version
# -----------------------------------------
#        Moudule        | Version|rgb_addr|
# -----------------------------------------
#   LCD1602 Module      |  V1.0  | 0x60   |
# -----------------------------------------
#   LCD1602 Module      |  V1.1  | 0x6B   |
# -----------------------------------------
#   LCD1602 RGB Module  |  V1.0  | 0x60   |
# -----------------------------------------
#   LCD1602 RGB Module  |  V2.0  | 0x2D   |
# -----------------------------------------

board = board = telemetrix.Telemetrix(com_port="COM4")
board.set_pin_mode_i2c()
lcd=DFRobot_RGBLCD1602(board)                               #create LCD object,specify col and row

'''
  @brief set RGB
  @param r  red   range(0-255)
  @param g  green range(0-255)
  @param b  blue  range(0-255)
'''
lcd.set_RGB(colorR, colorG, colorB)
time.sleep(1)
lcd.clear()    
lcd.set_RGB(0, 0, 0)
# Print a message to the LCD.
'''
  @brief output data to LCD to display
  @param arg output data
'''
lcd.print_out("hello, world!")
lcd.scroll_display_left()
time.sleep(1)
lcd.scroll_display_left()

time.sleep(1)
lcd.no_display()

time.sleep(1)
lcd.display()

i = 0

while True:
  i = i+1
  # set the cursor to column 0, line 1
  # (note: line 1 is the second row, since counting begins with 0):
  '''
    @brief set cursor position
    @param col columns optional range 0-15
    @param row rows optional range 0-1，0 is the first row, 1 is the second row
  '''
  lcd.set_cursor(0, 1)
  # print the number of seconds since reset:
  lcd.print_out(i)
  time.sleep(1)



import sys
import time

from telemetrix import telemetrix

# set arduino pins acoording to L298N motor driver outputs
ANALOG_PIN = 3  # enA
ANALOG_PIN = 9  # enB
DIGITAL_PIN = 4  # in1
DIGITAL_PIN = 5  # in2
DIGITAL_PIN = 6  # in3
DIGITAL_PIN = 7  # in4
SERVO_PIN = 12

# first motor controlled by (enA,in1,in2) -> (3, 4, 5) pins
# second motor controlled by (enB,in3,in4) -> (9, 6, 7) pins


class wheels():
    def __init__(self):
        # some globals
        # Create a Telemetrix instance.
        self.board = telemetrix.Telemetrix()
        # attach servo pin
        self.board.set_pin_mode_servo(SERVO_PIN, 900, 2000)
        # set speed of DC motors
        self.speed = 255  # min: 8 - max: 255

    def run(self):
        self.dcTest()
        self.servoTest()

    def set_speed(self):
        # setting same speed for both motors for sync movement (check current voltage for both motors to be sure they are sync )
        self.board.analog_write(3, self.speed)  # Analog Pin Write
        self.board.analog_write(9, self.speed)  # Analog Pin Write

    def car_forward(self):
        self.board.digital_write(4, 1)
        self.board.digital_write(5, 0)
        self.board.digital_write(6, 1)
        self.board.digital_write(7, 0)

    def car_backward(self):
        self.board.digital_write(4, 0)
        self.board.digital_write(5, 1)
        self.board.digital_write(6, 0)
        self.board.digital_write(7, 1)

    def car_left(self):
        self.board.digital_write(4, 1)
        self.board.digital_write(5, 0)
        self.board.digital_write(6, 0)
        self.board.digital_write(7, 1)

    def car_right(self):
        self.board.digital_write(4, 0)
        self.board.digital_write(5, 1)
        self.board.digital_write(6, 1)
        self.board.digital_write(7, 0)

    def stop(self):
        self.board.digital_write(4, 0)
        self.board.digital_write(5, 0)
        self.board.digital_write(6, 0)
        self.board.digital_write(7, 0)

    def servoTest(self):
        self.board.servo_write(SERVO_PIN, 0)
        time.sleep(1)
        self.board.servo_write(SERVO_PIN, 90)
        time.sleep(1)
        self.board.servo_write(SERVO_PIN, 180)
        time.sleep(1)
        self.board.servo_write(SERVO_PIN, 0)
        time.sleep(1)
        self.board.servo_detach(SERVO_PIN)

    def dcTest(self):
        self.set_speed()
        self.car_forward()
        time.sleep(3)
        self.stop()
        time.sleep(.2)
        self.car_backward()
        time.sleep(3)
        self.stop()
        time.sleep(.2)
        self.car_left()
        time.sleep(3)
        self.stop()
        time.sleep(.2)
        self.car_right()
        time.sleep(3)
        self.stop()
        time.sleep(.2)

    def shutDown(self):
        self.board.shutdown()


if __name__ == '__main__':
    wheels().run()



import sys
import time
from telemetrix import telemetrix

"""
/* This struct helps us constructing the I2C output based on data and control outputs.
   Because the LCD is set to 4-bit mode, 4 bits of the I2C output are for the control outputs
   while the other 4 bits are for the 8 bits of data which are send in parts using the enable output.*/
"""


class OutputState(object):
    def __init__(self):
        self.rs = 0
        self.rw = 0
        self.E = 0
        self.Led = 0
        self.data = 0

    def GetLowData(self):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= self.E << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0x0F) << 4

        return buffer

    def GetHighData(self):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= self.E << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0xF0)

        return buffer


class LCD_I2C(object):
    def __init__(self, board, address=0x27, columns=16, rows=2):
        self.board = board
        self._address = address
        self._columnMax = columns - 1
        self._rowMax = rows - 1
        self._displayState = 0x00
        self._entryState = 0x00
        self._output = OutputState()

        board.set_pin_mode_i2c()
        board.i2c_write(self._address, [0])

    def begin(self):
        self.delayMiliseconds(50)  # Wait more than 40ms after powerOn.
        self.I2C_Write(0b00000000)  # Clear i2c adapter
        self.delayMiliseconds(50)  # Wait more than 40ms after powerOn.

        self.InitializeLCD()

    def backlight(self):
        self._output.Led = 1
        # Led pin is independent of LCD data and control lines.
        self.I2C_Write(0b00000000 | self._output.Led << 3)

    def noBacklight(self):
        self._output.Led = 0
        # Led pin is independent of LCD data and control lines.
        self.I2C_Write(0b00000000 | self._output.Led << 3)

    def clear(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00000001)
        self.delayMiliseconds(160)

    def home(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00000010)
        self.delayMiliseconds(160)

    # Part of Entry mode set
    def leftToRight(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState |= 1 << 1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def rightToLeft(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState &= ~(1 << 1)

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def autoscroll(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState |= 1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def noAutoscroll(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState &= ~1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Display control
    def display(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1 << 2

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noDisplay(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~(1 << 2)

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def cursor(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1 << 1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noCursor(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~(1 << 1)

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def blink(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noBlink(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Cursor or display shift
    def scrollDisplayLeft(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00011000)
        self.delayMiliseconds(37)

    # Part of Cursor or display shift
    def scrollDisplayRight(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00011100)
        self.delayMiliseconds(37)

    # Set CGRAM address
    def createChar(self, location, charmap=[]):
        self._output.rs = 0
        self._output.rw = 0

        location %= 8

        self.LCD_Write(0b01000000 | (location << 3))
        self.delayMiliseconds(37)

        for i in charmap:
            self.write(i)

        self.setCursor(0, 0)  # Set the address pointer back to the DDRAM

    # Set DDRAM address
    def setCursor(self, col, row):
        row_offsets = []
        row_offsets.extend([0x00, 0x40, 0x14, 0x54])
        self._output.rs = 0
        self._output.rw = 0

        if col > self._columnMax:
            col = self._columnMax  # sanity limits
        if row > self._rowMax:
            row = self._rowMax  # sanity limits

        newAddress = row_offsets[row] + col

        self.LCD_Write(0b10000000 | newAddress)
        self.delayMiliseconds(37)

    def write(self, character):
        self._output.rs = 1
        self._output.rw = 0

        self.LCD_Write(character)
        self.delayMiliseconds(41)

        return 1

    def InitializeLCD(self):
        # See HD44780U datasheet "Initializing by Instruction" Figure 24 (4-Bit Interface)
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(42)
        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(150)
        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(37)
        self.LCD_Write(0b00100000, True)  # Function Set - 4 bits mode
        self.delayMiliseconds(37)
        self.LCD_Write(0b00101000)  # Function Set - 4 bits(Still), 2 lines, 5x8 font
        self.delayMiliseconds(37)

        self.display()
        self.clear()
        self.leftToRight()

    def I2C_Write(self, output):
        self.board.i2c_write(self._address, [output])

    def LCD_Write(self, output, initialization=False):
        self._output.data = output

        self._output.E = True
        self.I2C_Write(self._output.GetHighData())
        self.delayMicroseconds(1)  # High part of enable should be >450 nS

        self._output.E = False
        self.I2C_Write(self._output.GetHighData())

        # During initialization, we only send half a byte
        if not initialization:
            # I think we need a delay between half byte writes,
            # but no sure how long it needs to be.
            self.delayMicroseconds(37)

            self._output.E = True
            self.I2C_Write(self._output.GetLowData())
            self.delayMicroseconds(1)  # High part of enable should be >450 nS

            self._output.E = False
            self.I2C_Write(self._output.GetLowData())
        # self.delayMicroseconds(37) # Some commands have different timing requirement,
        # so every command should handle its own delay after execution

    def delayMicroseconds(self, microsec):
        time.sleep(microsec * 1e-6)

    def delayMiliseconds(self, milisec):
        time.sleep(milisec * 1e-3)

    def print(self, text):
        for c in bytearray(text, 'ASCII'):
            self.write(c)



import sys
import time

from telemetrix import telemetrix

board = telemetrix.Telemetrix()
# board.shutdown()
board.r4_hard_reset()

time.sleep(2)
board = telemetrix.Telemetrix()

# board.shutdown()
board.r4_hard_reset()



import serial
import sys
import time


class NoJson:
    """
    This file sends the equivalent of

    {
          "count": 0,
          "type": "r",
          "device": "i2c",
          "address": 83,
          "register": 232,
          "big_data": 2048,
          "data":[1,2,3,4,5,6]
    }
    across the serial link.

    It takes 1.1878118515014648 seconds to send 1000 packets.
    """

    def __init__(self):
        self.serial_port = serial.Serial('/dev/ttyACM0', 115200,
                                         timeout=1, writeTimeout=0)
        self.start_time = 0
        print('open')

        while True:
            try:
                if self.serial_port.inWaiting():
                    # here we know the size of the packet
                    c = self.serial_port.read(14)

                    # retrieve the count within the data received
                    count = c[0:2]
                    x = int.from_bytes(count, "big")
                    if x == 0:
                        self.start_time = time.time()
                    if x == 999:
                        print(f'Elapsed time: {time.time() - self.start_time}')
                        sys.exit(0)
                else:
                    time.sleep(.000001)
                    # continue
            except OSError:
                pass


NoJson()



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import sys
import time

from telemetrix import telemetrix

"""
Setup a pin for output and fade its intensity
"""

# some globals
# make sure to select a PWM pin
DIGITAL_PIN = 6

# Create a Telemetrix instance.
board = telemetrix.Telemetrix()

# Set the DIGITAL_PIN as an output pin
board.set_pin_mode_analog_output(DIGITAL_PIN)

# When hitting control-c to end the program
# in this loop, we are likely to get a KeyboardInterrupt
# exception. Catch the exception and exit gracefully.

try:
    print('Fading up...')
    for i in range(255):
        board.analog_write(DIGITAL_PIN, i)
        time.sleep(.005)
    print('Fading down...')
    for i in range(255, -1, -1):
        board.analog_write(DIGITAL_PIN, i)
        time.sleep(.005)


except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)




"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 LCD_I2C - Arduino library to control a 16x2 LCD via an I2C adapter based on PCF8574
 * 2021-11-18 Brewmanz: make changes to also work for 20x4 LCD2004
 Copyright(C) 2020 Blackhack <davidaristi.0504@gmail.com>
 This program is free software : you can redistribute it and /or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.If not, see < https://www.gnu.org/licenses/>.
"""

import sys
import time
from telemetrix import telemetrix

"""
/* This struct helps us constructing the I2C output based on data and control outputs.
   Because the LCD is set to 4-bit mode, 4 bits of the I2C output are for the control outputs
   while the other 4 bits are for the 8 bits of data which are send in parts using the enable output.*/
"""


class OutputState(object):
    def __init__(self):
        self.rs = 0
        self.rw = 0
        self.E = 0
        self.Led = 0
        self.data = 0

    def GetLowData(self):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= self.E << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0x0F) << 4

        return buffer

    def GetHighData(self):
        buffer = self.rs
        buffer |= self.rw << 1
        buffer |= self.E << 2
        buffer |= self.Led << 3
        buffer |= (self.data & 0xF0)

        return buffer


class LCD_I2C(object):
    def __init__(self, board, address=0x27, columns=16, rows=2):
        self.board = board
        self._address = address
        self._columnMax = columns - 1
        self._rowMax = rows - 1
        self._displayState = 0x00
        self._entryState = 0x00
        self._output = OutputState()

        board.set_pin_mode_i2c()
        board.i2c_write(self._address, [0])

    def begin(self):
        self.delayMiliseconds(50)  # Wait more than 40ms after powerOn.
        self.I2C_Write(0b00000000)  # Clear i2c adapter
        self.delayMiliseconds(50)  # Wait more than 40ms after powerOn.

        self.InitializeLCD()

    def backlight(self):
        self._output.Led = 1
        self.I2C_Write(
            0b00000000 | self._output.Led << 3)  # Led pin is independent from LCD data and control lines.

    def noBacklight(self):
        self._output.Led = 0
        self.I2C_Write(
            0b00000000 | self._output.Led << 3)  # Led pin is independent from LCD data and control lines.

    def clear(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00000001)
        self.delayMiliseconds(160)

    def home(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00000010)
        self.delayMiliseconds(160)

    # Part of Entry mode set
    def leftToRight(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState |= 1 << 1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def rightToLeft(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState &= ~(1 << 1)

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def autoscroll(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState |= 1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Entry mode set
    def noAutoscroll(self):
        self._output.rs = 0
        self._output.rw = 0

        self._entryState &= ~1

        self.LCD_Write(0b00000100 | self._entryState)
        self.delayMiliseconds(37)

    # Part of Display control
    def display(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1 << 2

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noDisplay(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~(1 << 2)

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def cursor(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1 << 1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noCursor(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~(1 << 1)

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def blink(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState |= 1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Display control
    def noBlink(self):
        self._output.rs = 0
        self._output.rw = 0

        self._displayState &= ~1

        self.LCD_Write(0b00001000 | self._displayState)
        self.delayMiliseconds(37)

    # Part of Cursor or display shift
    def scrollDisplayLeft(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00011000)
        self.delayMiliseconds(37)

    # Part of Cursor or display shift
    def scrollDisplayRight(self):
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00011100)
        self.delayMiliseconds(37)

    # Set CGRAM address
    def createChar(self, location, charmap=[]):
        self._output.rs = 0
        self._output.rw = 0

        location %= 8

        self.LCD_Write(0b01000000 | (location << 3))
        self.delayMiliseconds(37)

        for i in charmap:
            self.write(i)

        self.setCursor(0, 0)  # Set the address pointer back to the DDRAM

    # Set DDRAM address
    def setCursor(self, col, row):
        row_offsets = []
        row_offsets.extend([0x00, 0x40, 0x14, 0x54])
        self._output.rs = 0
        self._output.rw = 0

        if (col > self._columnMax):
            col = self._columnMax  # sanity limits
        if (row > self._rowMax):
            row = self._rowMax  # sanity limits

        newAddress = row_offsets[row] + col

        self.LCD_Write(0b10000000 | newAddress)
        self.delayMiliseconds(37)

    def write(self, character):
        self._output.rs = 1
        self._output.rw = 0

        self.LCD_Write(character)
        self.delayMiliseconds(41)

        return 1

    def InitializeLCD(self):
        # See HD44780U datasheet "Initializing by Instruction" Figure 24 (4-Bit Interface)
        self._output.rs = 0
        self._output.rw = 0

        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(42)
        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(150)
        self.LCD_Write(0b00110000, True)
        self.delayMiliseconds(37)
        self.LCD_Write(0b00100000, True)  # Function Set - 4 bits mode
        self.delayMiliseconds(37)
        self.LCD_Write(0b00101000)  # Function Set - 4 bits(Still), 2 lines, 5x8 font
        self.delayMiliseconds(37)

        self.display()
        self.clear()
        self.leftToRight()

    def I2C_Write(self, output):
        self.board.i2c_write(self._address, [output])

    #        Wire.beginTransmission(_address)
    #        Wire.write(output)
    #        Wire.endTransmission()

    def LCD_Write(self, output, initialization=False):
        self._output.data = output

        self._output.E = True
        self.I2C_Write(self._output.GetHighData())
        self.delayMicroseconds(1)  # High part of enable should be >450 nS

        self._output.E = False
        self.I2C_Write(self._output.GetHighData())

        # During initialization we only send half a byte
        if (not (initialization)):
            self.delayMicroseconds(
                37)  # I think we need a delay between half byte writes, but no sure how long it needs to be.

            self._output.E = True
            self.I2C_Write(self._output.GetLowData())
            self.delayMicroseconds(1)  # High part of enable should be >450 nS

            self._output.E = False
            self.I2C_Write(self._output.GetLowData())
        # self.delayMicroseconds(37) # Some commands have different timing requirement,
        # so every command should handle its own delay after execution

    def delayMicroseconds(self, microsec):
        time.sleep(microsec * 1e-6)

    def delayMiliseconds(self, milisec):
        time.sleep(milisec * 1e-3)

    def print(self, text):
        for c in bytearray(text, 'ASCII'):
            self.write(c)



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys
import time
from telemetrix import telemetrix

"""
This example sets up and control an ADXL345 i2c accelerometer.
It will continuously print data the raw xyz data from the device.
"""


# the call back function to print the adxl345 data
def the_callback(data):
    """

    :param data: [pin_type, Device address, device read register, x data pair, y data pair, z data pair]
    :return:
    """
    print(data)


def adxl345(my_board):
    # setup adxl345
    # device address = 83
    my_board.set_pin_mode_i2c()

    # set up power and control register
    my_board.i2c_write(83, [45, 0])
    time.sleep(.1)
    my_board.i2c_write(83, [45, 8])
    time.sleep(.1)

    # set up the data format register
    my_board.i2c_write(83, [49, 8])
    time.sleep(.1)
    my_board.i2c_write(83, [49, 3])
    time.sleep(.1)

    # read_count = 20
    while True:
        # read 6 bytes from the data register
        try:
            my_board.i2c_read(83, 50, 6, the_callback)
            time.sleep(.1)

        except (KeyboardInterrupt, RuntimeError):
            my_board.shutdown()
            sys.exit(0)


board = telemetrix.Telemetrix()
try:
    adxl345(board)
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)



#!/usr/bin/env python3

import sys
import time
from telemetrix import telemetrix
from i2c_lcd_pcf8574 import LcdI2c


"""
This example sets up and control an pcf8574 i2c LCD display.
"""


def pcf8574(my_board):
    lcd = LcdI2c(my_board)
    lcd.begin()
    lcd.clear()
    lcd.createChar(0, [0b00000,
    0b10001,
    0b00000,
    0b00000,
    0b10001,
    0b01110,
    0b00000,
    0b00000])
    lcd.createChar(1, [0b01110,
    0b01010,
    0b01110,
    0b00100,
    0b10101,
    0b10101,
    0b01110,
    0b00100])
    lcd.write(1)
    lcd.write(0)
    time.sleep(2)
    # Flashing the backlight
    for i in range(2):
        lcd.noBacklight()
        time.sleep(1)
        lcd.backlight()
        time.sleep(1)
    lcd.cursor()
    lcd.blink()
    lcd.print("     Hello")  # You can make spaces using well... spaces
    time.sleep(2)
    lcd.noBlink()
    lcd.setCursor(5, 1)  # Or setting the cursor in the desired position.
    lcd.print("World!")
    lcd.delayMiliseconds(500)

    # Flashing the backlight
    for i in range(5):
        lcd.backlight()
        lcd.delayMiliseconds(500)
        lcd.noBacklight()
        lcd.delayMiliseconds(500)

    lcd.backlight()
    lcd.clear()
    lcd.delayMiliseconds(500)


board = telemetrix.Telemetrix()
try:
    pcf8574(board)
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)



#!/usr/bin/env python3

from setuptools import setup


setup(
    name='telemetrix',
    packages=['telemetrix'],
    install_requires=['pyserial'],

    version='0.1.2',
    description="telemetrix phase1 client and server",

    author='Alan Yorinks',
    author_email='MisterYsLab@gmail.com',
    url='https://github.com/MrYsLab/telemetrix',
    download_url='https://github.com/MrYsLab/telemetrix',
    keywords=['telemetrix', 'Arduino', 'Protocol', 'Python'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)




"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew

 Modified by Ahto Moorast:
 This example uses two analog read callback methods
 1. cb_analog_input_read_avg()
 Takes 'average_count' amount of samples from a specified pin and returns average value.
 Without currenty knowing how to pass arguments via callback function, the average sample count is controlled via global variables.

 2. cb_analog_input_read_pin()
 Returns a real-time value of the requested pin at request and nothing else. Mutes the callbacks right after.

 NOTE: To measure A15, one must modify Telemetrix4Arduino's MAX_ANALOG_PINS_SUPPORTED to 16 and reflash.
 Because on Mega2560 there's 16x analog pins A0 - A15.
 Line 525: #define MAX_ANALOG_PINS_SUPPORTED 16  //A0 - A15
"""

import sys
import time
import threading
from telemetrix import telemetrix
from matplotlib import pyplot as plot

pinout = {}  # Leave empty
# Dictionary to hold analog pins' raw ADC values.
ADC_results = {
    0: 0,  # pin 0 -> "A0" : 0V
    1: 0,
    2: 0,
    3: 0,
    4: 0,
    5: 0,
    6: 0,
    7: 0,
    8: 0,
    9: 0,
    10: 0,
    11: 0,
    12: 0,
    13: 0,
    14: 0,
    15: 0
}
active_pin = None  # pin used for reporting
average_count = 5  # number of samples to average
average_counter = 0  # used in callback

# Digital pins based on Arduino Digital pin numbering.
# When number is used for digital_ functions, the number is according to digital pins.  (13 = D13 != A13)
output_pins = {
    "d_LED": 13,  # Internal LED on MEGA
}

# When number is used for analog_ functions, the number is according to analog pins ??  (13 = A13 != D13)
# Alternatively use the digital pin numberation instead.  54 = D54 = A0
analog_input_pins = {
    "A0": 0,
    "A1": 1,
    "A2": 2,
    "A3": 3,
    "A4": 4,
    "A5": 5,
    "A6": 6,
    "A7": 7,
    "A8": 8,
    "A9": 9,
    "A10": 10,
    "A11": 11,
    "A12": 12,
    "A13": 13,
    "A14": 14,
    "A15": 15,
}

# merge all pins to one dictionary for easier referencing
pinout = analog_input_pins | output_pins

led_state = 1  # for this example

# Using Semitec 103JT-025 10kR NTC Thermistor
R1_temp = 10  # kR


# Mega has 10bits, thus 0 - 1023
def convert_analog(raw_reading: int, Vs: int = 5):
    return (raw_reading * (Vs / 1023))


# def convert_to_temperature(voltage_reading: int):


## ================================================================================================================

# specific class with Telemetrix interface
class Test:
    def __init__(self, com_port=None, arduino_instance_id=1):
        self.board = telemetrix.Telemetrix(com_port, arduino_instance_id)
        self.read_analog_timeout = 3  # in seconds.
        self.cb_flag = 0  # callback filter flag. Only do stuff when requested.

        # Configure pins specific to this example
        self.configure_pins()  # Configure test pin definitions. Assumes pins dictionary is well defined

    # Analog input callback function
    def cb_analog_input_read_avg(self, data):
        if (self.cb_flag):
            global average_count
            global average_counter
            global active_pin

            if (active_pin == data[1]):  # check that the callback is at the desired pin
                if (average_counter > 0):  # if there are still samples to be collected
                    average_counter -= 1
                    # print(f'{average_counter} Value {data[2]}')
                    ADC_results[data[1]] += data[
                        2]  # accumulate the measured value at the corresponding analog pin index
                else:  # calculate the average
                    # print(f'results: {ADC_results[data[1]]} | avg_cnt: {average_count}')
                    ADC_results[data[1]] = ADC_results[data[1]] / average_count
                    # print(f'new results: {ADC_results[data[1]]} | avg_cnt: {average_count}')
                    self.cb_flag = 0  # now reset the flag
        # print(f'Spamming callback from Type: {data[0]} Pin: {data[1]} Value: {data[2]}') # debug

    # Analog input callback function
    def cb_analog_input_read_pin(self, data):
        '''
        This implementation guarantees, only the pin of interest is updated at request.
        :param data: required by Telemetrix
        :return: currently nothing
        '''
        if (
                self.cb_flag == 1):  # Filter. Only act when requested. Otherwise callback is empty and terminates quickly.
            global active_pin  # using global variable...
            if (active_pin == data[1]):  # check that the callback is at the desired pin
                self.cb_flag = 0  # state that the callback has reached
                # print(f'Stored value: Pin Type: {data[0]} Pin: {data[1]} Value: {data[2]}')
                ADC_results[data[1]] = data[
                    2]  # Store the measured value at the corresponding analog pin index
                self.board.disable_analog_reporting(data[1])  # stop callbacks asap
                time.sleep(0.2)
        # print(f'callback Type: {data[0]} Pin: {data[1]} Value: {data[2]}') # debug
        # return data  # not needed at the moment as no way of getting the return from callback ?

    def configure_pins(self):
        # If you set the mode for multiple pins, you may wish to add a short delay after setting each pin.
        # https: // mryslab.github.io / telemetrix / pin_modes /

        # Outputs
        for x in output_pins:  # iterating through outputs to configure them as outputs in Arduino
            self.board.set_pin_mode_digital_output(output_pins[x])
            time.sleep(.02)
        print(f'# Digital output set done')

        '''
        # Analog inputs
        # This seems unnecessary to be done at startup. Since during read_analog, this pretty much repeats per pin.
        # To set up a 'read_all'-like functionality, it would be best to set_pin_mode_analog_input here for all Ax pins
        print("# Setting analog inputs:")
        for x in analog_input_pins:
            # Once pin_mode is set, callbacks are automatically enabled.
            self.board.set_pin_mode_analog_input(analog_input_pins[x], callback=self.cb_analog_input_read_avg)
            time.sleep(.02)
        '''
        self.board.set_analog_scan_interval(200)
        print(f'# Analog input set done')

        self.board.disable_all_reporting()

    def read_analog(self, a_pin_name: str, avg_cnt=5, delay=0.2):
        '''
        :param a_pin_name: string of a pin name written in the dictionary pinout
        :param delay:   default 0 seconds. Time stalled until reading a measurement.
        :param avg_cnt: is only used when cb_analog_input_read_avg() callback is used.
        :return: ([0]return value, [1]return message)
                 [0] - 0-5V - Specific pin's converted value of raw 10bit ADC value;
                 -1 - cb_flag is not reset during timeout
                 -2 - Reading callback not reached - something wrong with calling the callback;
                 [1] "status message")
        '''
        time.sleep(
            delay)  # wait for the delay before reading measurement. use >0.1 when making first measurements.
        pin = pinout[a_pin_name]  # get the pin number from dictionary

        # callback filtering
        global active_pin
        global average_count
        global average_counter
        active_pin = pin  # so that callback can acknowledge the pin under interest
        average_count = avg_cnt
        average_counter = avg_cnt
        self.cb_flag = 1
        ADC_results[pin] = 0  # reset the old measurement

        print(f'# Reading: {a_pin_name}, A{pin}')

        # Setting any analog pin causes callback to be triggered for that pin automatically.
        # self.board.set_pin_mode_analog_input(pin, callback=self.cb_analog_input_read_pin)  # to do single pin reading.
        self.board.set_pin_mode_analog_input(pin,
                                             callback=self.cb_analog_input_read_avg)  # average multiple reads
        time.sleep(0.1)

        time_now = time.time()  # used to do timeout checks.
        while (self.cb_flag == 1):  # callback check. flag is reset in callback.
            if time.time() <= time_now + self.read_analog_timeout:  # implement some timeout
                pass
            else:
                print("analog read timeout")
                return (-1, "analog read timeout")

        if (self.cb_flag == 0):  # check if callback had been reached
            # result is stored in ADC_results dictionary via callback function.
            rslt = convert_analog(ADC_results[pin])
            return (rslt, 'A' + str(pin) + ': ' + str(ADC_results[pin]))
        else:
            print("read_analog callback not reached")
            return (0, "read_analog callback not reached")

    def LED_blink(self, dir_delay: int = 0.5):
        '''
        Call this as a separate Thread !
        :param dir_delay: Time to wait until direction is changed
        :return:
        '''
        global led_state
        time_now = time.time()  # get time when function was called
        while True:  # infinite loop
            if (time.time() > (time_now + dir_delay)):  # When timeout has reached
                time_now = time.time()  # update the start of timer
                self.board.digital_write(pinout["d_LED"], led_state)
                # print(f"Blinking LED {led_state}")
                led_state ^= 1
            pass  # else do nothing


# ===============================================================================================================
# ===============================================================================================================

# Create a Telemetrix instance.
# board = telemetrix.Telemetrix()
test = Test()  # Creates an ServOCP test instance
print(f'# Telemetrix instance creation successful\n')
time.sleep(1)

# background task
print("# LED Blink")
# set 'daemon = True' to terminate function when main program exits.
led = threading.Thread(target=test.LED_blink, args=(0.5,), daemon=True)
led.start()

while True:

    try:
        print("# Read single ADCs")
        print(f'Test acknowledges: {test.read_analog("A15")}')  # A6

        time.sleep(2)
        print("# ADC Result contents")
        print(ADC_results)

        print("# Test script finished")

    except KeyboardInterrupt:
        test.board.shutdown()
        sys.exit(0)

test.board.shutdown()
sys.exit(0)



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

"""
This file is contains the class to control a PCA9685 i2c servo driver.
The code is loosely based upon the Adafruit library located at:
https://github.com/adafruit/Adafruit-PWM-Servo-Driver-Library
"""

import time

import pca9685_constants
from telemetrix import telemetrix


# noinspection GrazieInspection
class PCA9685:
    def __init__(self, i2c_address=pca9685_constants.PCA9685_I2C_ADDRESS,
                 osc_freq=pca9685_constants.FREQUENCY_OSCILLATOR,
                 i2c_port=0, prescale=0, board=None, position_min=150,
                 position_max=600):
        """

        :param i2c_address: i2c device address
        :param i2c_port: 0 or 1 for 2 port devices
        :param prescale: prescale value for external clock (optional)
        :param board: instance of Telemetrix. If none, one will be created.
        :param position_min: minimum servo position
        :param position_max: maximum servo position
        """
        self.i2c_address = i2c_address
        self.i2c_port = i2c_port
        self.prescale = prescale
        self.position_min = position_min
        self.position_max = position_max

        self.totempole = True
        self.pwm_data = None

        self.oscillator_freq = osc_freq

        self.position_per_degree = (self.position_max - self.position_min) // 180

        self.write_m_servo_number_pulse = []

        # create a telemetrix instance
        if board:
            self.board = board
        else:
            self.board = telemetrix.Telemetrix()

        # initialize the PCA9685
        self.board.set_pin_mode_i2c(self.i2c_port)
        self.reset()

        # set clock
        if self.prescale:
            self.set_ext_clk(self.prescale)
        else:
            # set a default pwm frequency
            self.set_pwm_freq(pca9685_constants.DEFAULT_PWM_FREQUENCY)

        # set default internal frequency
        self.set_oscillator_frequency(pca9685_constants.FREQUENCY_OSCILLATOR)

    def reset(self):
        """
        Sends a reset command to the PCA9685 chip over I2C
        """
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                pca9685_constants.MODE1_RESTART])
        time.sleep(0.01)

    def sleep(self):
        """
        Puts board into sleep mode
        First performs a read, and then processes return
        in _i2c_read_complete_sleep
        """
        self.board.i2c_read(self.i2c_address,
                            pca9685_constants.PCA9685_MODE1, 1,
                            self._i2c_read_complete_sleep)
        time.sleep(.5)

    def _i2c_read_complete_sleep(self, data):
        """
        Finish processing sleep command
        :param data: data from i2c device
        """
        # data is [i2c_read_report, port, number of bytes read, i2c address,
        #           device_register, data value, time_stamp]
        awake = data[5]
        # set sleep-bit high
        sleep_value = awake | pca9685_constants.MODE1_SLEEP

        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                sleep_value])
        time.sleep(.005)  # wait until cycle ends for sleep to be active

    def wakeup(self):
        """
        Wakes board from sleep.
        First performs a read, and then continues processing
        in the _i2c_read_complete_wake callback.
        """
        self.board.i2c_read(self.i2c_address,
                            pca9685_constants.PCA9685_MODE1, 1,
                            self._i2c_read_complete_wake)
        time.sleep(0.5)

    def _i2c_read_complete_wake(self, data):
        # data is [i2c_read_report, port, number of bytes read, i2c address,
        #           device_register, data value, time_stamp]
        sleep = data[5]
        # set sleep-bit high
        wake = sleep & ~pca9685_constants.MODE1_SLEEP

        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                wake])
        time.sleep(0.05)

    def set_ext_clk(self, prescale):
        """
        Sets EXTCLK pin to use the external clock
        First, performs a read, and then continues
        processing in the _i2c_read_complete_ext_clock callback.

        """
        self.prescale = prescale
        self.board.i2c_read(self.i2c_address,
                            pca9685_constants.PCA9685_MODE1, 1,
                            self._i2c_read_complete_ext_clock)
        time.sleep(0.5)

    def _i2c_read_complete_ext_clock(self, data):
        # data is [i2c_read_report, port, number of bytes read, i2c address,
        #           device_register, data value, time_stamp]
        old_mode = data[5]  # offset of 5 is the External Clock data value
        new_mode = (old_mode & ~pca9685_constants.MODE1_RESTART) | \
                   pca9685_constants.MODE1_SLEEP

        # go to sleep, turn off internal oscillator
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                new_mode])
        # this sets the SLEEP and EXTCLK bits of the mode1 register to
        # switch to use the external clock

        write_value = new_mode | pca9685_constants.MODE1_EXTCLK
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                write_value])

        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_PRESCALE,
                                                self.prescale])
        time.sleep(.005)

        # clear the SLEEP bit to start
        write_value = (new_mode & ~pca9685_constants.MODE1_SLEEP)
        write_value |= pca9685_constants.MODE1_RESTART
        write_value |= pca9685_constants.MODE1_AI

        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                write_value])

        time.sleep(0.05)

    def set_pwm_freq(self, freq):
        """
        Sets the PWM frequency for the entire chip, up to ~1.6 KHz
        Begins with a read, and then continues processing in the
        _i2c_read_complete_set_pwm_freq callback

        :param freq: Floating point frequency that we will attempt to match
        """
        if freq < 1:
            freq = 1
        if freq > 3500:
            freq = 3500

        prescale_value = ((self.oscillator_freq / (freq * 4096.0)) + 0.5) - 1

        if prescale_value < pca9685_constants.PCA9685_PRESCALE_MIN:
            prescale_value = pca9685_constants.PCA9685_PRESCALE_MIN
        if prescale_value > pca9685_constants.PCA9685_PRESCALE_MAX:
            prescale_value = pca9685_constants.PCA9685_PRESCALE_MAX

        self.prescale = int(prescale_value)

        self.board.i2c_read(self.i2c_address,
                            pca9685_constants.PCA9685_MODE1, 1,
                            self._i2c_read_complete_set_pwm_freq)

    def _i2c_read_complete_set_pwm_freq(self, data):
        old_mode = data[5]  # offset 5 is the reported mode value
        new_mode = (old_mode & ~pca9685_constants.MODE1_RESTART) | \
                   pca9685_constants.MODE1_SLEEP
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                new_mode])
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_PRESCALE,
                                                int(self.prescale)])
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                old_mode])
        time.sleep(0.005)

        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE1,
                                                old_mode |
                                                pca9685_constants.MODE1_RESTART |
                                                pca9685_constants.MODE1_AI])

    def set_output_mode(self, totempole=True):
        """
        Sets the output mode of the PCA9685 to either
        open drain or push pull / totempole.

        Begins with a read and completes the operation
        in the _i2c_read_complete_set_output callback

        Warning: LEDs with integrated zener diodes should
        only be driven in open drain mode.

        :param totempole: If true set to totempole, open drain if false.
        """
        self.totempole = totempole
        self.board.i2c_read(self.i2c_address,
                            pca9685_constants.PCA9685_MODE2, 1,
                            self._i2c_read_complete_set_output)
        time.sleep(0.5)

    def _i2c_read_complete_set_output(self, data):
        old_mode = data[5]
        if self.totempole:
            new_mode = old_mode | pca9685_constants.MODE2_OUTDRV
        else:
            new_mode = old_mode & ~pca9685_constants.MODE2_OUTDRV
        self.board.i2c_write(self.i2c_address, [pca9685_constants.PCA9685_MODE2,
                                                new_mode])

    def read_pre_scale(self):
        """
        Returns prescale value set in the device
        Begins with a read and completes the process in
        the _i2c_read_complete_read_prescale callback
        :return: prescale value
        """
        self.prescale = None
        self.board.i2c_read(self.i2c_address, pca9685_constants.PCA9685_PRESCALE, 1,
                            self._i2c_read_complete_read_prescale)
        time.sleep(0.05)

    def _i2c_read_complete_read_prescale(self, data):
        """
        data[5] = data value returned
        Modify this callback to notify your code when it
        is executed.
        :param data:
        :return:
        """
        self.prescale_read = True
        self.prescale = int(data[5])  # data value returned

    def get_pwm(self, num):
        """
        Gets the PWM output of one of the PCA9685 pins
        :param num: One of the PWM output pins (0 - 15)
        :return: requested PWM output value
        """
        self.pwm_data = None
        register = pca9685_constants.PCA9685_LED0_ON_L + 4 * num
        self.board.i2c_read(self.i2c_address,
                            register, 4,
                            self._i2c_read_complete_get_pwm)

    def _i2c_read_complete_get_pwm(self, data):
        """
        Modify this method to notify your code when the read
        completes
        :param data[5]: pwm value
        """
        self.pwm_data = data[5]

    def set_pwm(self, num, on, off):
        """
        Sets the PWM output of one of the PCA9685 pins
        :param num: One of the PWM output pins (0 - 15)
        :param on: Point in the 4096-part cycle to turn the PWM output ON
        :param off: Point in the 4096-part cycle to turn the PWM output OFF
        :return:
        """
        self.board.i2c_write(self.i2c_address,
                             [pca9685_constants.PCA9685_LED0_ON_L + 4 * num, on & 0xff,
                              on >> 8, off & 0xff, off >> 8])

    def set_pin(self, num, value, invert=False):
        """
        Helper to set pin PWM output.
        Sets pin without having to deal with on/off tick
        placement and properly handles a zero value as completely off and
        4095 as completely on.  Optional invert parameter supports inverting the
        pulse for sinking to ground.

        :param num: pwm output pin (0 - 15)
        :param value: The number of ticks out of 4096 to be active,
                      should be a value from 0 to 4095 inclusive.
        :param invert: Inverts the output
        """

        # Clamp value between 0 and 4095 inclusive.
        value = min(value, 4095)
        if invert:
            if value == 0:
                # Special value for signal fully on.
                self.set_pwm(num, 4096, 0)
            elif value == 4095:
                # Special value for signal fully off
                self.set_pwm(num, 0, 4096)
            else:
                self.set_pwm(num, 0, 4095 - value)
        else:
            if value == 4095:
                # set fully on
                self.set_pwm(num, 4096, 0)
            elif value == 0:
                # set fully off
                self.set_pwm(num, 0, 4096)
            else:
                self.set_pwm(num, 0, value)

    def get_oscillator_frequency(self):
        """
        Get the internally tracked oscillator frequency.
        :return: oscillator frequency
        """
        return self.oscillator_freq

    def set_oscillator_frequency(self, frequency):
        """
        Set the internally tracked oscillator frequency.
        :param frequency: frequency
        """
        self.oscillator_freq = frequency

    def set_angle(self, servo_num, angle):
        self.set_pwm(servo_num, 0, (angle * self.position_per_degree) + self.position_min)


p = PCA9685()
time.sleep(0.1)
PIN=15
while True:
    for x in range(p.position_min, p.position_max):
        p.set_pwm(PIN, 0, x)
    time.sleep(.5)
    for x in range(p.position_max, p.position_min, -1):
        p.set_pwm(PIN, 0, x)
    time.sleep(.5)

    p.set_angle(PIN, 0)
    time.sleep(1)
    p.set_angle(PIN, 90)
    time.sleep(1)
    p.set_angle(PIN, 180)
    time.sleep(1)
    p.set_angle(PIN, 0)
    time.sleep(1)



import sys
import time

from telemetrix import telemetrix

# LEDs
DIGITAL_PIN1 = 6
DIGITAL_PIN2 = 9
DIGITAL_PIN3 = 10
DIGITAL_PIN4 = 11


# Create a Telemetrix instance.
board = telemetrix.Telemetrix()

# Set the DIGITAL_PINs as output pins
board.set_pin_mode_analog_output(DIGITAL_PIN1)
board.set_pin_mode_analog_output(DIGITAL_PIN2)
board.set_pin_mode_analog_output(DIGITAL_PIN3)
board.set_pin_mode_analog_output(DIGITAL_PIN4)

fade_values =[0, 65, 125, 255]
led_pins = [6, 9, 10, 11]

while True:
    try:
        for i in fade_values:
            for pin in led_pins:
                board.analog_write(pin, i)
            time.sleep(200)
    except KeyboardInterrupt:
        board.shutdown()


"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import sys
import time

from telemetrix import telemetrix

"""
Monitor a digital input pin
"""

"""
Setup a pin for digital input and monitor its changes
"""

# Set up a pin for analog input and monitor its changes
DIGITAL_PIN = 12  # arduino pin number

# Callback data indices
CB_PIN_MODE = 0
CB_PIN = 1
CB_VALUE = 2
CB_TIME = 3


def the_callback(data):
    """
    A callback function to report data changes.
    This will print the pin number, its reported value and
    the date and time when the change occurred

    :param data: [pin, current reported value, pin_mode, timestamp]
    """
    date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[CB_TIME]))
    print(f'Pin Mode: {data[CB_PIN_MODE]} Pin: {data[CB_PIN]} Value: {data[CB_VALUE]} Time Stamp: {date}')


def digital_in(my_board, pin):
    """
     This function establishes the pin as a
     digital input. Any changes on this pin will
     be reported through the call back function.

     :param my_board: a telemetrix instance
     :param pin: Arduino pin number
     """

    # set the pin mode
    my_board.set_pin_mode_digital_input(pin, the_callback)
    # time.sleep(1)
    # my_board.disable_all_reporting()
    # time.sleep(4)
    # my_board.enable_digital_reporting(12)

    # time.sleep(3)
    # my_board.enable_digital_reporting(pin)
    # time.sleep(1)

    print('Enter Control-C to quit.')
    # my_board.enable_digital_reporting(12)
    try:
        while True:
            time.sleep(.0001)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)


board = telemetrix.Telemetrix()

try:
    digital_in(board, DIGITAL_PIN)
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

"""
This is a very simple program to read the temperature using the BMe280 device.
The code is a loosely based port of the Sparkfun BME280 library and SPI example.
https://github.com/sparkfun/SparkFun_BME280_Arduino_Library

Note: the temperature reported may not be accurate. The intent of this program is to
provide examples of using the telemetrix API.
"""

import time
import sys
from telemetrix import telemetrix


class BME280Spi:
    """
    This class provides SPI control of a BME280
    """

    def __init__(self, chip_select_pin=10):
        """

        :param chip_select_pin: SPI chip select pin used for this device
        """
        # some "constants"
        self.BME_280_ID_REGISTER = 0xD0
        self.VALID_ID_VALUE = 96
        self.NORMAL_MODE = 3

        # temp calibration registers
        self.BME280_DIG_T1_LSB_REG = 0x88
        self.BME280_DIG_T1_MSB_REG = 0x89
        self.BME280_DIG_T2_LSB_REG = 0x8A
        self.BME280_DIG_T2_MSB_REG = 0x8B
        self.BME280_DIG_T3_LSB_REG = 0x8C
        self.BME280_DIG_T3_MSB_REG = 0x8D

        self.BME280_CTRL_MEAS_REG = 0xF4

        # temperature register
        self.BME280_TEMPERATURE_MSB_REG = 0xFA

        # temperature calibration data
        self.calibration_T1 = None
        self.calibration_T2 = None
        self.calibration_T3 = None

        self.chip_select_pin = chip_select_pin

        # instantiate telemetrix
        self.board = telemetrix.Telemetrix()

        # initialize the SPI interface
        self.board.set_pin_mode_spi([self.chip_select_pin])
        time.sleep(1)

        # initialize the device
        self.init_device()

        # continuously read the temperature
        self.get_temperature()

    def device_callback(self, data):
        """
        :param data: Report data sent from the board resident telemetrix sketch

        Indices into data:
            chip_select_pin = 1
            register = 2
            number bytes returned = 3
            data bytes 4...
        """
        # Index to spi register value in data
        CHIP_SELECT = 1
        REGISTER = 2
        # NUMBER_OF_BYTES_RETURNED = 3
        START_OF_DATA = 4

        if data[CHIP_SELECT] == self.chip_select_pin:
            # chip ID handler
            if data[REGISTER] == self.BME_280_ID_REGISTER:
                if data[START_OF_DATA] == 96:
                    # print('Valid ID returned')
                    return
                else:
                    print(f'Invalid ID: {data[START_OF_DATA]}')

            # calibration data handlers
            elif data[REGISTER] == self.BME280_DIG_T1_MSB_REG:
                self.calibration_T1 = data[START_OF_DATA] << 8
            elif data[REGISTER] == self.BME280_DIG_T1_LSB_REG:
                self.calibration_T1 += data[START_OF_DATA]

                print(f't1 {self.calibration_T1}')

            elif data[REGISTER] == self.BME280_DIG_T2_MSB_REG:
                self.calibration_T2 = data[START_OF_DATA] << 8
            elif data[REGISTER] == self.BME280_DIG_T2_LSB_REG:
                self.calibration_T2 += data[START_OF_DATA]

                print(f't2 {self.calibration_T2}')

            elif data[REGISTER] == self.BME280_DIG_T3_MSB_REG:
                self.calibration_T3 = data[START_OF_DATA] << 8
            elif data[REGISTER] == self.BME280_DIG_T3_LSB_REG:
                self.calibration_T3 += data[START_OF_DATA]

                print(f't3 {self.calibration_T3}')

        # temperature data handler
            elif data[REGISTER] == self.BME280_TEMPERATURE_MSB_REG:
                print(f'temp data = {data}')
                raw_temperature = data[START_OF_DATA] << 12 or data[START_OF_DATA + 1] \
                                  << 4 or ((data[START_OF_DATA + 2] >> 4) & 0x0F)
                print(raw_temperature)
                time.sleep(.5)
                print('here')

                """
                
                var1 = (((raw_temperature >> 3) - (self.calibration_T1 << 1)) * \
                        self.calibration_T2) >> 11

                var2 = (((((raw_temperature >> 4) - (self.calibration_T1)) * (
                        (raw_temperature >> 4) - (self.calibration_T1))) >> 12) *
                        self.calibration_T1) >> 14

                t_fine = var1 + var2

                output = (t_fine * 5 + 128) >> 8

                output = output / 100
                """
                # print(
                #     f'Temperature - degrees C: {output}  degrees F:
                #    {(output * 9) / 5 + 32}')

    def init_device(self):

        print('Initializing BME280...')

        # get id byte from bme280

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME_280_ID_REGISTER,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        # read calibration values for temperature
        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T1_MSB_REG,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T1_LSB_REG,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T2_MSB_REG,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T2_LSB_REG,
                                     1, call_back=self.device_callback)

        time.sleep(.3)
        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T3_MSB_REG,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_DIG_T3_LSB_REG,
                                     1, call_back=self.device_callback)
        time.sleep(.3)

        # set measurement mode to normal
        self.board.spi_write_blocking(self.chip_select_pin, [self.BME280_CTRL_MEAS_REG,
                                                             163])
        # time.sleep(.3)

        # write8(BME280_REGISTER_CONTROL, 163)

        # finally reread the ID byte to make sure nothing has changed
        # self.board.spi_read_blocking(self.chip_select_pin,
        #                              self.BME_280_ID_REGISTER,
        #                              1, call_back=self.device_callback)
        # time.sleep(.3)

        print('BME280 successfully initialized!')

    def get_temperature(self):
        """
        Continuously retrieve temperature in degrees C.
        """

        while True:
            try:
                self.board.spi_read_blocking(self.chip_select_pin,
                                             self.BME280_TEMPERATURE_MSB_REG,
                                             3,
                                             call_back=self.device_callback)
                time.sleep(2)

            except KeyboardInterrupt:
                self.board.shutdown()
                sys.exit(0)


CS_PIN = 10

bme = BME280Spi(CS_PIN)



"""
 Copyright (c) 2023 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

"""
This is a program to read a BME280 humidity, pressure and temperature sensor.
The code is a loosely based upon https://github.com/Zanduino/BME280

"""

import time
import sys
from telemetrix import telemetrix


class BME280Spi:
    """
    This class provides SPI control of a BME280
    """

    def __init__(self, chip_select_pin=10):
        """

        :param chip_select_pin: SPI chip select pin used for this device
        """

        # BME280 Register Constants
        
        
        self.BM280_CHIPID_REG = 0xD0 # ID register
        self.BM280_CHIPID = 0x60 # expected ID

        self.BM280_SOFT_RESET_REG = 0xE0  # Reset when 0xB6 is written here
        self.BM280_RESET_CODE = 0xB6

        self.BM280_CONTROL_HUMID_REG = 0xF2
        
        self.BM280_STATUS_REG = 0xF3
        
        self.BM280_CONTROL_REG = 0xF4
        
        self.BM280_CONFIG_REG = 0xF5
        
        self.BM280_PRESSURE_DATA_REG = 0xF7
        
        self.BM280_TEMP_DATA_REG = 0xFA        
        
        self.BM280_HUMID_DATA_REG = 0xFD
        
        # calibration registers
        
        # temperature
        self.BM280_T1_REG = 0x88
            
        self.BM280_T2_REG = 0x8A
        
        self.BM280_T3_REG = 0x8C
        
        # pressure
        self.BM280_P1_REG = 0x8E
        
        self.BM280_P2_REG = 0x90
        
        self.BM280_P3_REG = 0x92
        
        self.BM280_P4_REG = 0x94
        
        self.BM280_P5_REG = 0x96
        
        self.BM280_P6_REG = 0x98
        
        self.BM280_P7_REG = 0x9A
        
        self.BM280_P8_REG = 0x9C
        
        self.BM280_P9_REG = 0x9E
        
        # humidity
        self.BM280_H1_REG = 0xA1
        
        self.BM280_H2_REG = 0xE1
        
        self.BM280_H3_REG = 0xE3
        
        self.BM280_H4_REG = 0xE4
        
        self.BM280_H5_REG = 0xE5
        
        self.BM280_H6_REG = 0xE7

        

        # temperature calibration data
        self.calibration_T1 = None
        self.calibration_T2 = None
        self.calibration_T3 = None

        self.chip_select_pin = chip_select_pin

        # instantiate telemetrix
        self.board = telemetrix.Telemetrix(arduino_wait=4)

        # initialize the SPI interface
        self.board.set_pin_mode_spi([self.chip_select_pin])
        time.sleep(.3)

        # self.board.spi_set_format(2, 1, 0)

        # initialize the device
        self.init_device()

        time.sleep(1)

        # continuously read the temperature
        self.get_temperature()

    def device_callback(self, data):
        """
        :param data: Report data sent from the board resident telemetrix sketch

        Indices into data:
            chip_select_pin = 1
            register = 2
            number bytes returned = 3
            data bytes 4...
        """
        # Index to spi register value in data
        CHIP_SELECT = 1
        REGISTER = 2
        # NUMBER_OF_BYTES_RETURNED = 3
        START_OF_DATA = 4

        if data[CHIP_SELECT] == self.chip_select_pin:
            # chip ID handler
            if data[REGISTER] == self.BME_280_ID_REGISTER:
                if data[START_OF_DATA] == 96:
                    # print('Valid ID returned')
                    return
                else:
                    print(f'Invalid ID: {data[START_OF_DATA]}')

            # calibration data handlers
            elif data[REGISTER] == self.BME280_REGISTER_DIG_T1:
                self.calibration_T1 = (data[START_OF_DATA] >> 8) | (data[START_OF_DATA +
                                                                       1] << 8)

            elif data[REGISTER] == self.BME280_REGISTER_DIG_T2:
                self.calibration_T2 = (data[START_OF_DATA] >> 8) | (data[START_OF_DATA +
                                                                       1] << 8)

            elif data[REGISTER] == self.BME280_REGISTER_DIG_T3:
                self.calibration_T3 = (data[START_OF_DATA] >> 8) | (data[START_OF_DATA +
                                                                       1] << 8)

                print(f'cal t1: {self.calibration_T1}  cal t2: {self.calibration_T2} '
                      f'cal t3: {self.calibration_T3} ')

        # temperature data handler
            elif data[REGISTER] == self.BME280_TEMPERATURE_REG:
                print(data)
                raw_data = bytearray()
                raw_data.append(data[START_OF_DATA])
                raw_data.append(data[START_OF_DATA + 1])
                raw_data.append(data[START_OF_DATA + 2])

                raw_value = int.from_bytes(raw_data, "big")

                print(f'raw value {raw_value}')

                adjusted_value = raw_value >> 4

                print(f'adjusted: {adjusted_value}')

                var1 = (adjusted_value / 8) - (self.calibration_T1 * 2)
                var1 = (var1 * self.calibration_T2) / 2048
                var2 = (adjusted_value / 16) - self.calibration_T1
                var2 = ((var2 * var2) / 4096) * (self.calibration_T3 / 16384)

                print(f'var1 = {var1}  var2 = {var2}')

                t_fine = var1 + var2

                T = ((t_fine * 5 + 128) / 256) / 100

                print(f'Temperature - degrees C: {T:.2f}  degrees F: '
                      f'{((T * 9) / 5 + 32):.2f}')

    def init_device(self):

        print('Initializing BME280...')

        # get id byte from bme280

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME_280_ID_REGISTER,
                                     1, call_back=self.device_callback)
        # read calibration values for temperature
        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_REGISTER_DIG_T1,
                                     2, call_back=self.device_callback)
        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_REGISTER_DIG_T2,
                                     2, call_back=self.device_callback)

        self.board.spi_read_blocking(self.chip_select_pin,
                                     self.BME280_REGISTER_DIG_T3,
                                     2, call_back=self.device_callback)

        # set measurement mode to normal
        self.board.spi_write_blocking(self.chip_select_pin, [self.BME280_CTRL_MEAS_REG,
                                                             163])
        print('BME280 successfully initialized!')

    def get_temperature(self):
        """
        Continuously retrieve temperature in degrees C.
        """

        while True:
            try:
                self.board.spi_read_blocking(self.chip_select_pin,
                                             self.BME280_TEMPERATURE_REG,
                                             3,
                                             call_back=self.device_callback)
                time.sleep(.5)

            except KeyboardInterrupt:
                self.board.shutdown()
                sys.exit(0)


CS_PIN = 10

bme = BME280Spi(CS_PIN)



import time
from telemetrix import telemetrix


class Main:
    def __init__(self):
        self.board = telemetrix.Telemetrix()
        # init password buttons callback
        for i in range(10, 23):
            self.board.set_pin_mode_digital_input_pullup(i, callback=self.ringBack)
            time.sleep(.02)

    def ringBack(self, state):
        print(state)


if __name__ == '__main__':
    game = Main()
    while True:
        time.sleep(0.01)



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

"""
This file contains the register and other values to control the PCA9685 i2c Servo
Driver. The code is based upon the Adafruit library located at:
https://github.com/adafruit/Adafruit-PWM-Servo-Driver-Library
"""

# REGISTER ADDRESSES
PCA9685_MODE1 = 0x00  # Mode Register 1
PCA9685_MODE2 = 0x01  # Mode Register 2
PCA9685_SUBADR1 = 0x02  # I2C-bus subaddress 1
PCA9685_SUBADR2 = 0x03  # I2C-bus subaddress 2
PCA9685_SUBADR3 = 0x04  # I2C-bus subaddress 3
PCA9685_ALLCALLADR = 0x05  # LED All Call I2C-bus address
PCA9685_LED0_ON_L = 0x06  # LED0 on tick, low byte
PCA9685_LED0_ON_H = 0x07  # LED0 on tick, high byte
PCA9685_LED0_OFF_L = 0x08  # LED0 off tick, low byte
PCA9685_LED0_OFF_H = 0x09  # LED0 off tick, high byte

#  etc all 16:  LED15_OFF_H = 0x45
PCA9685_ALLLED_ON_L = 0xFA  # load all the LEDn_ON registers, low 
PCA9685_ALLLED_ON_H = 0xFB  # load all the LEDn_ON registers, high 
PCA9685_ALLLED_OFF_L = 0xFC  # load all the LEDn_OFF registers, low
PCA9685_ALLLED_OFF_H = 0xFD  # load all the LEDn_OFF registers,high
PCA9685_PRESCALE = 0xFE  # Prescaler for PWM output frequency
PCA9685_TESTMODE = 0xFF  # defines the test mode to be entered

#  MODE1 bits
# noinspection GrazieInspection
MODE1_ALLCAL = 0x01  # respond to LED All Call I2C-bus address
MODE1_SUB3 = 0x02  # respond to I2C-bus subaddress 3
MODE1_SUB2 = 0x04  # respond to I2C-bus subaddress 2
MODE1_SUB1 = 0x08  # respond to I2C-bus subaddress 1
MODE1_SLEEP = 0x10  # Low power mode. Oscillator off
MODE1_AI = 0x20  # Auto-Increment enabled
MODE1_EXTCLK = 0x40  # Use EXTCLK pin clock 
MODE1_RESTART = 0x80  # Restart enabled
# MODE2 bits
MODE2_OUTNE_0 = 0x01  # Active LOW output enable input
MODE2_OUTNE_1 = 0x02  # Active LOW output enable input - high impedance
MODE2_OUTDRV = 0x04  # totem pole structure vs open-drain
MODE2_OCH = 0x08  # Outputs change on ACK vs STOP
MODE2_INVRT = 0x10  # Output logic state inverted 

PCA9685_I2C_ADDRESS = 0x40  # Default PCA9685 I2C Slave Address
FREQUENCY_OSCILLATOR = 27000000  # Int. osc. frequency in datasheet

PCA9685_PRESCALE_MIN = 3  # minimum prescale value
PCA9685_PRESCALE_MAX = 255  # maximum prescale value

DEFAULT_PWM_FREQUENCY = 50



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

"""
This is a very simple program to read the BME280 id byte using an SPI interface.
It will change the SPI frequency and then do another read.
"""

import time
import sys
from telemetrix import telemetrix

CHIP_SELECT_PIN = 10

BME280_ID_REGISTER = 0xD0

NUM_BYTE_TO_READ = 1  # the BME280 ID is a single byte

VALID_ID_RETURN = 96


def device_callback(data):

    print(data)
    # Index to spi register value in data
    REGISTER = 2

    # Index to returned ID value in data
    ID_VALUE = 4

    if data[REGISTER] == BME280_ID_REGISTER:
        if data[ID_VALUE] == 96:
            print('Valid ID returned')
        else:
            print(f'Invalid ID: {data[ID_VALUE]}')


# instantiate an instance of Telemetrix
board = telemetrix.Telemetrix()

# Enable the SPI interface using pin 10 as the Chip Select Pin
board.set_pin_mode_spi([CHIP_SELECT_PIN])

print('Retrieving ID byte')

board.spi_read_blocking(CHIP_SELECT_PIN, BME280_ID_REGISTER,  NUM_BYTE_TO_READ,
                        call_back=device_callback)
time.sleep(1)
print("Changing SPI settings an then rereading ID Byte")
board.spi_set_format(8, 1, 0)
time.sleep(1)
board.spi_read_blocking(CHIP_SELECT_PIN, BME280_ID_REGISTER,  1,  call_back=device_callback)
time.sleep(1)
board.shutdown()
sys.exit(0)




"""
 Copyright (c) 2023 Pierre Jeunesse All rights reserved.

 https://github.com/PierreJeunesse

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

"""
This example sets up and control an MCP9600 i2c Thermocouple.
The structure is the same as MCP9600_basic_demo.ino provided by the manufacturer Seeed.
It will continuously print data from the device.
"""
import sys
import time

from telemetrix import telemetrix


# the call back function to print the MCP9600 data
def the_callback(data):
    """

    :param data: [Device address, device read register, data]
    :return:
    """

    first_byte = data[5]
    sec_byte = data[6]
    Temp_ = ((first_byte << 8) | sec_byte) / 16

    # Display result
    print(Temp_)


def mpc9600(my_board):
    # setup mpc9600
    # device address = 96
    my_board.set_pin_mode_i2c()  # #define DEFAULT_IIC_ADDR  0X60 (in Seeed_MCO9600.h)

    # err_t sensor_basic_config() {
    #     err_t ret = NO_ERROR;
    #     CHECK_RESULT(ret, sensor.set_filt_coefficients(FILT_MID)); (in Seeed_MCO9600.cpp)
    #           IIC_read_byte(THERM_SENS_CFG_REG_ADDR, &therm_cfg_data) (in Seeed_MCO9600.cpp)
    #               THERM_SENS_CFG_REG_ADDR : 0X5 => 5 (in Seeed_MCO9600.h)
    #               FILT_MID : 4 (in Seeed_MCO9600.h)
    my_board.i2c_write(96, [5, 4])
    time.sleep(1)

    #     CHECK_RESULT(ret, sensor.set_cold_junc_resolution(COLD_JUNC_RESOLUTION_0_25));
    #           same logic as above
    my_board.i2c_write(96, [6, 1 << 7])
    time.sleep(1)

    #     CHECK_RESULT(ret, sensor.set_ADC_meas_resolution(ADC_14BIT_RESOLUTION));
    my_board.i2c_write(96, [6, 1 << 5])
    time.sleep(1)

    #     CHECK_RESULT(ret, sensor.set_burst_mode_samp(BURST_32_SAMPLE));
    my_board.i2c_write(96, [6, 0 << 2])
    time.sleep(1)

    #     CHECK_RESULT(ret, sensor.set_sensor_mode(NORMAL_OPERATION));
    my_board.i2c_write(96, [6, 0])
    time.sleep(1)

    while True:
        # read 2 bytes from the data register
        try:
            my_board.i2c_read_restart_transmission(96, 0, 2, the_callback, i2c_port=0,
                                                   write_register=True)
            time.sleep(0.5)

        except (KeyboardInterrupt, RuntimeError):
            my_board.shutdown()
            sys.exit(0)


board = telemetrix.Telemetrix()
try:
    mpc9600(board)
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)



"""
 Copyright (c) 2021 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import sys
import time

from telemetrix import telemetrix

"""
Run a motor continuously without acceleration
"""

# Create a Telemetrix instance.
board = telemetrix.Telemetrix(ip_address='192.168.2.112')
# create an accelstepper instance for a TB6600 motor driver
motor = board.set_pin_mode_stepper(interface=1, pin1=5, pin2=4)
# if you are using a 28BYJ-48 Stepper Motor with ULN2003
# comment out the line above and uncomment out the line below.
# motor = board.set_pin_mode_stepper(interface=4, pin1=5, pin2=4, pin3=14, pin4=12)


# set the max speed and speed
board.stepper_set_max_speed(motor, 900)
board.stepper_set_speed(motor, 800)
# run the motor
board.stepper_run_speed(motor)

# keep application running
while True:
    try:
        time.sleep(.0002)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)
